import meteva
import numpy as np
import pandas as pd
import xarray as xr
import datetime
import collections
import copy


# 实现任意纬度分类的函数
def score_df(df, method, s = None,g=None,gll_dict = None,plot = None,excel_path = None,first = True,**kwargs):
    '''

    :param df:
    :param method:
    :param s:
    :param g:
    :param gll_dict:
    :param plot:
    :param excel_path:
    :param first:  first参数不是面对用户的，是程序自我递归调用时用到的参数
    :param kwargs:
    :return:
    '''
    method_name = method.__name__

    method_mid = meteva.perspact.get_middle_method(method)
    column_list = meteva.perspact.get_middle_columns(method_mid)
    score_method_with_mid = meteva.perspact.get_score_method_with_mid(method)
    df0 = meteva.base.sta_data(df)

    column_df = df.columns
    if not set(column_list) <= set(column_df):
        print("input pandas.DataFrame must contains columns in list of "+ str(column_list) + " for mem." + method_name + " caculation")
        return None,None

    if s is not None:
        if "member" in s.keys():
            if isinstance(s["member"],list):
                members = s["member"]
            else:
                members = [s["member"]]
            df0  = df0.loc[df0['member'].isin(members)]
            s.pop("member")


        if "lat" in s.keys() or "lon" in s.keys():
            region_ids = df0["id"]
            max_id = np.max(region_ids)
            min_id = np.min(region_ids)
            max_ = max(max_id, abs(min_id))
            if max_ < 1000000:
                lat = np.round(region_ids / 1000)
                lon = region_ids - lat * 1000
            else:
                lat = np.round(region_ids / 10000)
                lon = region_ids - lat * 10000
                lon /= 10
                lat /= 10
            df0["lon"] =lon
            df0["lat"] =lat


    df1 = meteva.base.sele_by_dict(df0, s)
    if len(df1.index) ==0:
        print("所选参数范围内没有检验数据")
        return None,None

    if method_name.find("ob_fo_") >= 0 and first:
        df_ob = copy.deepcopy(df1)
        df_ob["member"] = "OBS"
        if method_mid.__name__ == "hfmc":
            df_ob["H"] = df1["H"] + df1["M"]
            df_ob["F"] = 0
            df_ob["M"] = 0
            df_ob["C"] = df1["F"] + df1["C"]
        if method_mid.__name__ =="tmmsss":
            df_ob["T"] = df1["T"]
            df_ob["MY"] = df1["MX"]
        if method_mid.__name__ == "cscs":
            df_ob["CY"] = df1["CX"]
            df_ob["SY"] = df1["SX"]
            pass
        df_ob.drop_duplicates(subset=None, keep='first', inplace=True)
        df1 = pd.concat([df_ob,df1],axis=0)
        if g is None:
            g = ["member"]

        if "member" not in g:
            print("统计"+method_name+"时，参数g中必须包含“member")
            return

    g_time = ["time","year","month","day","hour","minute","dtime","xun","hou",
              "ob_time","ob_year","ob_month","ob_day","ob_hour","ob_minute","ob_xun","ob_hou",
              "day_hour","ob_day_hour","year_month","ob_year_month"]

    if g is None:
        g = [g]
        gll_dict = None
        gll0 = None
    else:

        #将g转成列表形式
        if isinstance(g,str):
            g = [g]

        #gll_dict 初始化包含所有分类维度
        if gll_dict is None:
            gll_dict = {}
            for gg in range(len(g)):
                gll_dict[g[gg]] = None

        #对每个维度的分类方式进行赋值
        #print(g)
        for gg in range(len(g)):
            if g[gg] not in gll_dict.keys() or  gll_dict[g[gg]] is None:

                if g[gg] in g_time:
                    _,gll = meteva.base.group(df1,g = g[gg])
                    gll.sort()
                else:
                    #为了保持原有排序，不用group函数
                    groups = df1[g[gg]]
                    groups = groups.drop_duplicates(keep="first")
                    gll = groups.values.tolist()
                    if not  isinstance(gll[0],str):
                        if g[gg] =="level":
                            gll.sort(reverse=True)
                        else:
                            gll.sort()

                gll_dict[g[gg]] = gll

        # 将分组方式统一成单层列表，或者两层列表
        gll_dict1 = {}
        for gg in range(len(g)):
            list_list = gll_dict[g[gg]]
            has_list = False
            for list1 in list_list:
                if  isinstance(list1,list):
                    has_list = True
            if has_list:
                list_list1 = []
                for list1 in list_list:
                    if isinstance(list1, list):
                        list_list1.append(list1)
                    else:
                        list_list1.append([list1])
            else:
                list_list1 = list_list
            gll_dict1[g[gg]] = list_list1
        gll_dict = gll_dict1

        #取出第一个分类维度的分类方式备用
        gll0 = gll_dict[g[0]]

    g0 = g[0]
    df1_list, gll = meteva.base.group(df1, g=g0, gll=gll0)
    if len(g) == 1:
        score_list = []
        gll_i_dict ={}
        score1 = None
        for i in range(len(df1_list)):
            if method_mid == meteva.method.tmmsss:
                tmmsss_array = df1_list[i][column_list].values
                mid_array = tmmsss_array[0, :]
                for j in range(1, tmmsss_array.shape[0]):
                    mid_array = meteva.method.tmmsss_merge(mid_array, tmmsss_array[j, :])

            elif method_mid == meteva.method.tems:
                tems_array = df1_list[i][column_list].values
                mid_array = tems_array[0, :]
                for j in range(1, tems_array.shape[0]):
                    mid_array = meteva.method.tems_merge(mid_array, tems_array[j, :])
            else:
                mid_list = []
                for column in column_list:
                    mid = np.sum(df1_list[i][column])
                    mid_list.append(mid)
                mid_array = np.array(mid_list)
                if method_mid == meteva.method.hnh:
                    mid_array = mid_array.reshape(10,2)
            score1 = score_method_with_mid(mid_array)
            score_list.append(score1)
            if gll is None:
                gll_str = "None"
            else:
                gll_str = str(gll[i])
            gll_i_dict[gll_str] = i


        if gll0 is None:
            score_list_with_iv = score_list
        else:
            score_list_with_iv = []
            for j in range(len(gll0)):
                gll_str = str(gll0[j])
                if gll_str in gll_i_dict.keys():
                    score2 = score_list[gll_i_dict[gll_str]]
                else:
                    score2 = score1 * 0 + meteva.base.IV
                score_list_with_iv.append(score2)

        score_array = np.array(score_list_with_iv)
        if method_name.find("ob_fo_")>=0:
             score_array = score_array[...,1,0]

        if plot =="line" or plot =="bar" or plot =="barh":
            meteva.base.plot_tools.plot_bar(plot,score_array,name_list_dict=gll_dict,**kwargs)
        if plot =="mesh" or plot =="contour":
            meteva.base.plot_tools.mesh_contourf(plot,score_array, name_list_dict=gll_dict, **kwargs)
        if plot =="lineh":
            meteva.base.plot_tools.lineh(score_array,name_list_dict=gll_dict,**kwargs)
        return  score_array,gll_dict
    else:
        if len(g) == 2:
            g_left = g[1]
        else:
            g_left = g[1:]

        gll_i_dict ={}
        score_all_list = []
        score_array = None
        for i in range(len(gll)):
            score_array,gll_dict_new = score_df(df1_list[i], method,g = g_left, gll_dict = gll_dict,plot = None,first = False)
            score_all_list.append(score_array)
            gll_str = str(gll[i])
            gll_i_dict[gll_str] = i
            # if "member" in gll_dict.keys():
            #     if "OBS" not in gll_dict["member"]:
            #         gll_dict["member"] = gll_dict_new["member"]  # 为ob_fo_ 函数增加OBS列

        score_list_with_iv = []
        for j in range(len(gll0)):
            gll_str = str(gll0[j])
            if gll_str in gll_i_dict.keys():
                score2 = score_all_list[gll_i_dict[gll_str]]
            else:
                score2 = score_array * 0 + meteva.base.IV
            score_list_with_iv.append(score2)
        score_all_array = np.array(score_list_with_iv)


        if plot =="line" or plot =="bar" or plot == "barh":
            meteva.base.plot_tools.plot_bar(plot, score_all_array, name_list_dict=gll_dict, **kwargs)
        if plot =="mesh" or plot =="contour":
            meteva.base.plot_tools.mesh_contourf(plot,score_all_array, name_list_dict=gll_dict, **kwargs)
        if plot =="lineh":
            meteva.base.plot_tools.lineh( score_all_array, name_list_dict=gll_dict, **kwargs)

        if excel_path is not None:
            meteva.base.write_array_to_excel(score_all_array,excel_path,gll_dict)

        return score_all_array,gll_dict

def sele_by_dict(ds0,s):
    if s is None:
        return ds0
    else:
        ob_keys =[]
        not_ob_keys = []
        for key in s.keys():
            if key.find("ob_")>=0:
                ob_keys.append(key)
            else:
                not_ob_keys.append(key)

        #收集非观测时间相关的时间选取参数
        s_not_ob = {}
        #收集和dtime维度相关的选取参数
        dtime_s = ds0.dtime.values
        for key in not_ob_keys:
            list0 = s[key]
            if not isinstance(list0, list):
                list0 = [list0]
            if key =="dtime":
                dtime_s = list(set(dtime_s) & set(list0))
                if len(dtime_s) ==0:
                    return None
                dtime_s.sort()
                not_ob_keys.remove(key)
            elif key =="dtime_range":
                list1 = []
                for value in dtime_s:
                    if value>=s[key][0] and value<= s[key][1]:
                        list1.append(value)
                if len(list1) == 0:
                    return None
                else:
                    list1.sort()
                    dtime_s = list1
                not_ob_keys.remove(key)


        #收集和time维度相关的选取参数
        time_s = ds0.time.values
        for key in not_ob_keys:
            list0 = s[key]
            if not isinstance(list0, list):
                list0 = [list0]
            if key =="time":
                values_s1 = []
                for value in list0:
                    values_s1.append(meteva.base.all_type_time_to_time64(value))
                fo_times = pd.Series(0, index=time_s)
                time_s = time_s[fo_times.index.isin(values_s1)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="year":
                fo_times = pd.Series(0, index=time_s)
                time_s = time_s[fo_times.index.year.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="month":
                fo_times = pd.Series(0, index=time_s)
                time_s = time_s[fo_times.index.month.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="xun":
                fo_times = pd.Series(0, index=time_s)
                mons = fo_times.index.month.astype(np.int16)
                days = fo_times.index.day.astype(np.int16)
                xuns = np.ceil(days / 10).values.astype(np.int16)
                xuns[xuns > 3] = 3
                xuns += (mons - 1) * 3
                xuns = pd.Series(xuns)
                time_s = time_s[xuns.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="hou":
                fo_times = pd.Series(0, index=time_s)
                mons = fo_times.index.month.astype(np.int16)
                days = fo_times.index.day.astype(np.int16)
                hous = np.ceil(days / 5).values.astype(np.int16)
                hous[hous > 6] = 6
                hous += (mons - 1) * 6
                hous = pd.Series(hous)
                time_s = time_s[hous.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="day":
                days_list = []
                time0 = datetime.datetime(1900, 1, 1, 0, 0)
                seconds = 3600 * 24
                for day0 in list0:
                    day0 = meteva.base.tool.time_tools.all_type_time_to_datetime(day0)
                    day = int((day0 - time0).total_seconds() // seconds)
                    days_list.append(day)
                days = (time_s - meteva.base.all_type_time_to_time64(time0)) // np.timedelta64(1, "D")
                days = pd.Series(days)
                time_s = time_s[days.isin(days_list)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="dayofyear":
                fo_times = pd.Series(0, index=time_s)
                time_s = time_s[fo_times.index.dayofyear.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="hour":
                fo_times = pd.Series(0, index=time_s)
                time_s = time_s[fo_times.index.hour.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="minute":
                fo_times = pd.Series(0, index=time_s)
                time_s = time_s[fo_times.index.minute.isin(list0)]
                if len(time_s) == 0:
                    return None
                not_ob_keys.remove(key)
            elif key =="time_range":
                start_time = meteva.base.all_type_time_to_time64(list0[0])
                end_time = meteva.base.all_type_time_to_time64(list0[1])
                list1 = []
                for value in time_s:
                    if value>=start_time and value<= end_time:
                        list1.append(value)
                if len(list1) == 0:
                    return None
                else:
                    dtime_s = np.array(list1)
                not_ob_keys.remove(key)

        #根据观测时间参和可用时效确定实际可用的起报时间
        time_exp = None
        dtime_exp = None
        if len(ob_keys)>0:
            time_exp,dtime_exp = np.meshgrid(time_s,dtime_s)
            time_exp = time_exp.flatten()
            dtime_exp = dtime_exp.flatten()
            dtime_exp_64 = dtime_exp * np.timedelta64(1, 'h')
            obtimes_exp = time_exp + dtime_exp_64
            for key in ob_keys:
                list0 = s[key]
                if not isinstance(list0, list):
                    list0 = [list0]
                if key =="ob_time":
                    values_s1 = []
                    for value in list0:
                        values_s1.append(meteva.base.all_type_time_to_time64(value))
                    ob_times = pd.Series(0, index=obtimes_exp)
                    index = ob_times.index.isin(values_s1)
                    time_exp = time_exp[index]
                    obtimes_exp = obtimes_exp[index]
                    dtime_exp = dtime_exp[index]
                    if len(obtimes_exp) == 0:
                        return None
                elif key =="ob_year":
                    fo_times = pd.Series(0, index=time_s)
                    time_s = time_s[fo_times.index.year.isin(list0)]
                    if len(dtime_s) == 0:
                        return None

                elif key =="ob_month":
                    fo_times = pd.Series(0, index=time_s)
                    time_s = time_s[fo_times.index.month.isin(list0)]
                    if len(time_s) == 0:
                        return None

                elif key =="ob_xun":
                    fo_times = pd.Series(0, index=time_s)
                    mons = fo_times.index.month.astype(np.int16)
                    days = fo_times.index.day.astype(np.int16)
                    xuns = np.ceil(days / 10).values.astype(np.int16)
                    xuns[xuns > 3] = 3
                    xuns += (mons - 1) * 3
                    xuns = pd.Series(xuns)
                    time_s = time_s[xuns.isin(list0)]
                    if len(time_s) == 0:
                        return None

                elif key =="ob_hou":
                    fo_times = pd.Series(0, index=time_s)
                    mons = fo_times.index.month.astype(np.int16)
                    days = fo_times.index.day.astype(np.int16)
                    hous = np.ceil(days / 5).values.astype(np.int16)
                    hous[hous > 6] = 6
                    hous += (mons - 1) * 6
                    hous = pd.Series(hous)
                    time_s = time_s[hous.isin(list0)]
                    if len(time_s) == 0:
                        return None
                elif key =="ob_day":
                    days_list = []
                    time0 = datetime.datetime(1900, 1, 1, 0, 0)
                    seconds = 3600 * 24
                    for day0 in list0:
                        day0 = meteva.base.tool.time_tools.all_type_time_to_datetime(day0)
                        day = int((day0 - time0).total_seconds() // seconds)
                        days_list.append(day)
                    days = (time_s - meteva.base.all_type_time_to_time64(time0)) // np.timedelta64(1, "D")
                    days = pd.Series(days)
                    time_s = time_s[days.isin(days_list)]
                    if len(time_s) == 0:
                        return None
                elif key =="ob_dayofyear":
                    fo_times = pd.Series(0, index=time_s)
                    time_s = time_s[fo_times.index.dayofyear.isin(list0)]
                    if len(time_s) == 0:
                        return None
                    not_ob_keys.remove(key)
                elif key =="ob_hour":
                    fo_times = pd.Series(0, index=time_s)
                    time_s = time_s[fo_times.index.hour.isin(list0)]
                    if len(time_s) == 0:
                        return None
                elif key =="ob_minute":
                    fo_times = pd.Series(0, index=time_s)
                    time_s = time_s[fo_times.index.minute.isin(list0)]
                    if len(time_s) == 0:
                        return None

                elif key =="ob_time_range":
                    start_time = meteva.base.all_type_time_to_time64(list0[0])
                    end_time = meteva.base.all_type_time_to_time64(list0[1])
                    list1 = []
                    for value in time_s:
                        if value>=start_time and value<= end_time:
                            list1.append(value)
                    if len(list1) == 0:
                        return None
                    else:
                        dtime_s = np.array(list1)

            time_s = list(set(time_exp))
            time_s.sort()
            dtime_s = list(set(dtime_exp))
            dtime_s.sort()
        s_not_ob["dtime"] = dtime_s
        s_not_ob["time"] = time_s

        #收集其它维度相关的参数
        for key in not_ob_keys:
            list0 = s[key]
            if not isinstance(list0, list):
                list0 = [list0]
            s_not_ob[key] =list0
        ds1 = ds0.sel(s_not_ob)

        #将和观测时间不对应的部分重置为nan
        if time_exp is not None:
            df = pd.DataFrame({"time":time_exp,"dtime":dtime_exp,"data0":1})
            df.sort_values(by=["time","dtime"],inplace=True)
            df_new = pd.DataFrame(df["data0"].values, index=[df["time"].values,df["dtime"].values],columns=["data0"])
            df_new.index.names = ["time","dtime"]
            s_xr =  df_new.to_xarray()
            ds2 = ds1.where(s_xr.data0==1)  # 通过s_xr中的nan将ds1中的相应维度重置为0。
        else:
            ds2 = ds1

        return ds2

def get_gll(ds0,g):

    if g in ds0.coords:
        return ds0[g].values
    elif g.find("ob_")>=0:
        time_s = ds0["time"].values
        dtime_s = ds0["dtime"].values

        time_exp, dtime_exp = np.meshgrid(time_s, dtime_s)
        time_exp = time_exp.flatten()
        dtime_exp = dtime_exp.flatten()
        dtime_exp_64 = dtime_exp * np.timedelta64(1, 'h')
        obtimes_exp = time_exp + dtime_exp_64
        obtime_s = list(set(obtimes_exp))
        obtime_s.sort()
        if g =="ob_time":
            return obtime_s
        elif g == "ob_year":
            obtimes = pd.Series(0, index=obtime_s)
            grouped_dict = dict(list(obtimes.groupby(obtimes.index.year)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "ob_month":
            obtimes = pd.Series(0, index=obtime_s)
            grouped_dict = dict(list(obtimes.groupby(obtimes.index.month)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "ob_dayofyear":
            obtimes = pd.Series(0, index=obtime_s)
            grouped_dict = dict(list(obtimes.groupby(obtimes.index.dayofyear)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "ob_day":
            obtimes = pd.Series(0, index=obtime_s)
            time0 = datetime.datetime(1900, 1, 1, 0, 0)
            indexs = (obtimes.index - time0) // np.timedelta64(1, "D")
            grouped_dict = dict(list(obtimes.groupby(indexs)))
            keys = list(grouped_dict.keys())
            gll = []
            for key in keys:
                gll.append([time0 + datetime.timedelta(days=key)])
            return gll
        elif g == "ob_hour":
            obtimes = pd.Series(0, index=obtime_s)
            grouped_dict = dict(list(obtimes.groupby(obtimes.index.hour)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "ob_minute":
            obtimes = pd.Series(0, index=obtime_s)
            grouped_dict = dict(list(obtimes.groupby(obtimes.index.minute)))
            keys = list(grouped_dict.keys())
            return keys
    else:
        time_s = ds0["time"].values

        if g == "year":
            times = pd.Series(0, index=time_s)
            grouped_dict = dict(list(times.groupby(times.index.year)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "month":
            times = pd.Series(0, index=time_s)
            grouped_dict = dict(list(times.groupby(times.index.month)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "dayofyear":
            times = pd.Series(0, index=time_s)
            grouped_dict = dict(list(times.groupby(times.index.dayofyear)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "day":
            times = pd.Series(0, index=time_s)
            time0 = datetime.datetime(1900, 1, 1, 0, 0)
            indexs = (times.index - time0) // np.timedelta64(1, "D")
            grouped_dict = dict(list(times.groupby(indexs)))
            keys = list(grouped_dict.keys())
            gll = []
            for key in keys:
                gll.append([time0 + datetime.timedelta(days=key)])
            return gll
        elif g == "hour":
            times = pd.Series(0, index=time_s)
            grouped_dict = dict(list(times.groupby(times.index.hour)))
            keys = list(grouped_dict.keys())
            return keys
        elif g == "minute":
            times = pd.Series(0, index=time_s)
            grouped_dict = dict(list(times.groupby(times.index.minute)))
            keys = list(grouped_dict.keys())
            return keys


def sele_ob_time(ds0,ob_time):

    ob_time = meteva.base.all_type_time_to_time64(ob_time)
    time_s = ds0.time.values
    dtime_s = ds0.dtime.values
    ds_merge = None
    for dtime in dtime_s:
        time1 = ob_time -dtime * np.timedelta64(1, 'h')
        if time1 in time_s:
            ds1 = ds0.sel({"time":time1,"dtime":dtime})
            ds1 = ds1.drop("time")
            ds1.coords["ob_time"] = ob_time
            ds1 = ds1.expand_dims("dtime")
            ds1 = ds1.expand_dims("ob_time")
            if ds_merge is None:
                ds_merge = ds1
            else:
                ds_merge = ds_merge.merge(ds1)
    return ds_merge


def group(ds0,g = None,gll = None):

    if g is None:
        return [ds0],None
    elif g =="ob_time":
        gll = get_gll(ds0,g)
        ds_list = []
        for i in range(len(gll)):
            ds1 = sele_ob_time(ds0, ob_time = gll[i])
            ds_list.append(ds1)
        return ds_list, gll
    else:

        if gll is not None:
            ds_list = []
            for i in range(len(gll)):
                ds1 = sele_by_dict(ds0, s={g: gll[i]})
                ds_list.append(ds1)
            return ds_list, gll
        else:
            if g in ds0.coords:
                groups = list(ds0.groupby(g))
                valid_gll = []
                ds_list = []
                for i in range(len(groups)):
                    valid_gll.append(groups[i][0])
                    ds_list.append(groups[i][1])

                return ds_list,valid_gll
            else:
                gll = get_gll(ds0,g)
                ds_list = []
                for i in range(len(gll)):
                    ds1 = sele_by_dict(ds0, s={g: gll[i]})
                    ds_list.append(ds1)
                return ds_list, gll


def score_ds(ds,method,s = None,g = None,gll_dict = None,plot = None,first = True,**kwargs):
    method_name = method.__name__
    if method_name.find("ob_fo_") >= 0 and first:
        df1 = meteva.perspact.tran_middle_ds_to_df(ds)
        return score_df(df1,method,s = s,g = g,gll_dict=gll_dict,plot=plot,first = first,**kwargs)

    method_mid = meteva.perspact.get_middle_method(method)
    value_list = meteva.perspact.get_middle_columns(method_mid)
    score_method_with_mid = meteva.perspact.get_score_method_with_mid(method)

    for value in value_list:
        if not value in ds:
            print("input xarray.DataSet must contains columns in list of " + str(
            value_list) + " for mem." + method_name + " caculation")
            return None, None


    ds1 = sele_by_dict(ds, s)

    if g is None:
        g = [g]
        gll_dict = None
        gll0 = None
    else:

        # 将g转成列表形式
        if isinstance(g, str):
            g = [g]

        #gll_dict 初始化包含所有分类维度
        if gll_dict is None:
            gll_dict = {}
            for gg in range(len(g)):
                gll_dict[g[gg]] = None

        # 对每个维度的分类方式进行赋值
        for gg in range(len(g)):
            if g[gg] not in gll_dict.keys() or gll_dict[g[gg]] is None:
                gll_dict[g[gg]] = get_gll(ds1,g[gg])

        # 将分组方式统一成单层列表，或者两层列表
        gll_dict1 = {}
        for gg in range(len(g)):
            list_list = gll_dict[g[gg]]
            has_list = False
            for list1 in list_list:
                if isinstance(list1, list):
                    has_list = True
            if has_list:
                list_list1 = []
                for list1 in list_list:
                    if isinstance(list1, list):
                        list_list1.append(list1)
                    else:
                        list_list1.append([list1])
            else:
                list_list1 = list_list
            gll_dict1[g[gg]] = list_list1
        gll_dict = gll_dict1
        # 取出第一个分类维度的分类方式备用
        gll0 = gll_dict[g[0]]

    g0 = g[0]
    ds1_list, gll = group(ds1, g=g0,gll = gll0)

    if len(g) == 1:
        score_list = []
        gll_i_dict = {}
        score1 = None
        for i in range(len(ds1_list)):
            if method_mid == meteva.method.tmmsss:

                mid_values_list = []
                for column in value_list:
                    mid_values = ds1_list[i][column].values.flatten()
                    mid_values_list.append(mid_values)

                mid_all = np.array(mid_values_list)
                tmmsss_array = mid_all.T
                mid_array = tmmsss_array[0, :]
                for j in range(1, tmmsss_array.shape[0]):
                    mid_array = meteva.method.tmmsss_merge(mid_array, tmmsss_array[j, :])
            else:
                mid_list = []
                for column in value_list:
                    mid = np.nansum(ds1_list[i][column])
                    mid_list.append(mid)
                mid_array = np.array(mid_list)
            score1 = score_method_with_mid(mid_array)
            score_list.append(score1)
            if gll is None:
                gll_str = "None"
            else:
                gll_str = str(gll[i])
            gll_i_dict[gll_str] = i

        if gll0 is None:
            score_list_with_iv = score_list
        else:
            score_list_with_iv = []
            for j in range(len(gll0)):
                gll_str = str(gll0[j])
                if gll_str in gll_i_dict.keys():
                    score2 = score_list[gll_i_dict[gll_str]]
                else:
                    score2 = score1 * 0 + meteva.base.IV
                score_list_with_iv.append(score2)

        score_array = np.array(score_list_with_iv)

        if plot is not None: meteva.base.plot_tools.plot_bar(plot, score_array, name_list_dict=gll_dict, **kwargs)
        return score_array, gll_dict
    else:
        if len(g) == 2:
            g_left = g[1]
        else:
            g_left = g[1:]

        gll_i_dict = {}
        score_all_list = []
        score_array = None
        for i in range(len(gll)):
            score_array, _ = score_ds(ds1_list[i], method, g=g_left, gll_dict=gll_dict, plot=None)
            score_all_list.append(score_array)
            gll_str = str(gll[i])
            gll_i_dict[gll_str] = i

        score_list_with_iv = []
        for j in range(len(gll0)):
            gll_str = str(gll0[j])
            if gll_str in gll_i_dict.keys():
                score2 = score_all_list[gll_i_dict[gll_str]]
            else:
                score2 = score_array * 0 + meteva.base.IV
            score_list_with_iv.append(score2)
        score_all_array = np.array(score_list_with_iv)
        score_all_array[np.isnan(score_all_array)] = meteva.base.IV
        if plot is not None: meteva.base.plot_tools.plot_bar(plot, score_all_array, name_list_dict=gll_dict, **kwargs)

        return score_all_array, gll_dict


def score_xy_df(df_mid,method,s = None,g = None,gll_dict = None,save_path = None,plot = True,**kwargs):

    g_list = []
    if isinstance(g,list):
        if len(g)>1:
            print("score_xy_df函数暂时不支持多维分类功能")
            return
        else:
            g_list = g
    else:
        if g is not None:
            g_list = [g]
    #g_list.append("id")
    g_list.append("lon")
    g_list.append(("lat"))

    if s is not None:
        if "member" in s.keys():
            df_mid  = df_mid.loc[df_mid['member'].isin(s["member"])]
            s.pop("member")
        if "grade" in s.keys():
            df_mid = df_mid.loc[df_mid['grade']== s["grade"]]
            s.pop("grade")


    region_ids = df_mid["id"]
    max_id = np.max(region_ids)
    min_id = np.min(region_ids)
    max_ = max(max_id, abs(min_id))
    if max_ < 1000000:
        lat = np.round(region_ids / 1000)
        lon = region_ids - lat * 1000
    else:
        lat = np.round(region_ids / 10000)
        lon = region_ids - lat * 10000
        lon /= 10
        lat /= 10
    df_mid["lon"] =lon
    df_mid["lat"] =lat

    #print(df_mid)
    df_mid = meteva.base.sele_by_dict(df_mid, s)
    if len(df_mid.index)==0:
        print("no middle data had been seleted")
        return None
    score_array,g_dict =score_df(df_mid,method,g = g_list,gll_dict = gll_dict)

    g_time = ["time","year","month","day","hour","minute","dtime","xun","hou",
              "ob_time","ob_year","ob_month","ob_day","ob_hour","ob_minute","ob_xun","ob_hou"]

    # region_ids = g_dict["id"]
    # if region_ids is not None:
    #     region_ids = np.array(region_ids)
    # max_id = np.max(region_ids)
    # min_id = np.min(region_ids)
    # max_ = max(max_id,abs(min_id))
    # if max_ <1000000:
    #     lat = np.round(region_ids  / 1000)
    #     lon = region_ids - lat * 1000
    # else:
    #     lat = np.round(region_ids / 10000)
    #     lon = region_ids - lat * 10000
    #     lon /=10
    #     lat /=10
    grd_list = []
    gnames0 = None
    if len(g_list) == 2:
        # dict1 = {"level":df_mid["level"].values[0],"time":df_mid["time"].values[0],"dtime":df_mid["dtime"].values[0],
        #     "id": region_ids, "lon": lon, "lat": lat,  "score": score_array}
        # df = pd.DataFrame(dict1)
        # score_sta = meteva.base.sta_data(df)
        # score_grd = meteva.base.trans_sta_to_grd(score_sta)
        grd1 = xr.DataArray(score_array,coords=[("lon",g_dict["lon"]),("lat",g_dict["lat"])])
        score_grd = meteva.base.xarray_to_griddata(grd1)
        grd_list.append(score_grd)

    elif len(g_list) == 3:

        gnames0 = g_dict[g_list[0]]
        #print(gnames0)
        ng0 = len(gnames0)
        for i in range(ng0):
            grd1 = xr.DataArray(score_array[i,:], coords=[("lon", g_dict["lon"]), ("lat", g_dict["lat"])])
            score_grd = meteva.base.xarray_to_griddata(grd1)
            meteva.base.set_griddata_coords(score_grd,member_list=[gnames0[i]])
            grd_list.append(score_grd)


        # if g_list[0] =="grade":
        #     grd_list = []
        #     vmin = 10e30
        #     vmax = -10e30
        #     for i in range(ng0):
        #         name = gnames0[i]
        #         value = score_array[i,:]
        #         df = pd.DataFrame({"level": df_mid["level"].values[0], "time": df_mid["time"].values[0],
        #                            "dtime": df_mid["dtime"].values[0], "id": region_ids,
        #                            "lon": lon, "lat": lat, name: value})
        #         score_sta = meteva.base.sta_data(df)
        #
        #         score_grd = meteva.base.trans_sta_to_grd(score_sta)
        #         score_grd.values[score_grd.values == meteva.base.IV] = 0
        #         grd_list.append(score_grd)
        #
        #         # vmax1 = np.max(score_grd.values)
        #         # vmin1 = np.max(score_grd.values)
        #         # if vmax1 > vmax: vmax = vmax1
        #         # if vmin1 < vmin: vmin = vmin1
        #
        #     #meteva.base.plot_tools.plot_2d_grid_list(grd_list,save_path=save_path,vmax=vmax,vmin = vmin,**kwargs)
        #
        # elif g_list[0] in g_time:
        #     for i in range(ng0):
        #         #name = gnames0[i]
        #         value = score_array[i, :]
        #
        #         dict1 = {"id": region_ids, "lon": lon, "lat": lat,  "score": value}
        #         if "level" not in dict1:
        #             dict1["level"] = df_mid["level"].values[0]
        #         if "time" not in dict1:
        #             dict1["time"] = df_mid["time"].values[0]
        #         if "dtime" not in dict1:
        #             dict1["dtime"] = df_mid["dtime"].values[0]
        #         df = pd.DataFrame(dict1)
        #         score_sta = meteva.base.sta_data(df)
        #         score_grd = meteva.base.trans_sta_to_grd(score_sta)
        #         grd_list.append(score_grd)
        #         #title_list.append(g_list[0] + ":" + str(name))
        #     #meteva.base.plot_tools.plot_2d_grid_list(grd_list, save_path,title = title_list,type="mesh")
        # else:
        #     for i in range(ng0):
        #         name = gnames0[i]
        #         value = score_array[i,:]
        #         if g_list[0] == "member":
        #             df = pd.DataFrame({"level":df_mid["level"].values[0],"time":df_mid["time"].values[0],"dtime":df_mid["dtime"].values[0],"id":region_ids,
        #                                "lon": lon, "lat": lat, name: value})
        #         else:
        #             dict1 = {"id":region_ids,"lon":lon,"lat":lat,g_list[0]:name,"score":value}
        #             if "level" not in dict1:
        #                 dict1["level"] = df_mid["level"].values[0]
        #             if "time" not in dict1:
        #                 dict1["time"] = df_mid["time"].values[0]
        #             if "dtime" not in dict1:
        #                 dict1["dtime"] = df_mid["dtime"].values[0]
        #             df = pd.DataFrame(dict1)
        #         score_sta = meteva.base.sta_data(df)
        #
        #         score_grd = meteva.base.trans_sta_to_grd(score_sta)
        #         grd_list.append(score_grd)
        #     score_grd = meteva.base.concat(grd_list)
        #     #title_list = score_grd.coords[g_list[0]].values.tolist()
        #     #kwargs["title"]  = title
        #     #score_grd.values[score_grd.values == meteva.base.IV] = 0
        #     #meteva.base.plot_tools.plot_2d_grid_list(grd_list, save_path,title = title_list,type="mesh")


    if plot:
        grd_list_plot = []
        for i in range(len(grd_list)):
            score_grd = grd_list[i].copy()
            score_grd.values[score_grd.values == meteva.base.IV] = 0
            grd_list_plot.append(score_grd)
        if gnames0 is None:
            title_list = None
        else:
            title_list = []
            for i in range(len(gnames0)):
                title_list.append(str(gnames0[i]))
            kwargs["title"] = title_list
        meteva.base.plot_tools.plot_2d_grid_list(grd_list_plot, save_path = save_path, **kwargs)

    return grd_list,gnames0


def score_xy_df_delta(df_mid,method,reference_member,s = None,gll_dict = None,save_path = None,cmap ="me_bwr",**kwargs):
    '''
    绘制多个模式预报评分相对其中某一个模式的评分的正负技巧
    :param df_mid:
    :param method:
    :param reference_member:
    :param s:
    :param save_path:
    :param kwargs:
    :return:
    '''
    grd_list, gnames0 = score_xy_df(df_mid,method,s = s,g = "member",gll_dict=gll_dict,plot = False)
    grd_ref = None
    for i in range(len(gnames0)):
        if reference_member == gnames0[i]:
            grd_ref = copy.deepcopy(grd_list[i])
    if grd_ref is None:
        print(reference_member + " not exist in DataFrame of middle result")

    for i in range(len(grd_list)):
        grd_list[i].values -= grd_ref.values

    grd_list_plot = []
    for i in range(len(grd_list)):
        score_grd = grd_list[i].copy()
        score_grd.values[score_grd.values == meteva.base.IV] = 0
        grd_list_plot.append(score_grd)

    title_list = []
    for i in range(len(gnames0)):
        title_list.append(str(gnames0[i]))
    kwargs["title"] = title_list
    meteva.base.plot_tools.plot_2d_grid_list(grd_list_plot,  save_path = save_path,cmap=cmap, **kwargs)

    return grd_list,gnames0


def score_xz_df(df_mid,method,s = None,g = None,gll_dict = None,save_path = None,plot = True,**kwargs):
    g_list = []
    if isinstance(g,list):
        if len(g)>1:
            print("score_xz_df函数暂时不支持多维分类功能")
            return
        else:
            g_list = g
    else:
        if g is not None:
            g_list = [g]

    g_list.append("lon")
    g_list.append("level")
    if s is not None:
        if "member" in s.keys():
            df_mid  = df_mid.loc[df_mid['member'].isin(s["member"])]
            s.pop("member")
        if "grade" in s.keys():
            df_mid = df_mid.loc[df_mid['grade']== s["grade"]]
            s.pop("grade")


    #print(df_mid)

    ids = df_mid["id"] #所有格点的编号
    max_id = np.max(ids)
    min_id = np.min(ids)
    max_ = max(max_id, abs(min_id))
    if max_ < 1000000:
        lat = np.round(ids / 1000)
        lon = ids - lat * 1000
    else:
        lat = np.round(ids / 10000)
        lon = ids - lat * 10000
        lon /= 10
        lat /= 10
    df_mid["lon"] = lon
    df_mid["lat"] = lat
    df_mid = meteva.base.sele_by_dict(df_mid, s)

    score_array,g_dict =score_df(df_mid,method,g = g_list,gll_dict = gll_dict)

    coords = []
    for g1 in g_dict.keys():
        coords.append((g1,g_dict[g1]))
    da = xr.DataArray(score_array,coords=coords)
    grd = meteva.base.xarray_to_griddata(da)
    if plot:meteva.base.contourf_xz(grd, save_path=save_path,**kwargs)
    return grd


def score_xz_df_delta(df_mid,method,reference_member,s = None,gll_dict = None,save_path = None,cmap ="me_bwr",sup_title = "",**kwargs):
    '''
    绘制多个模式预报评分相对其中某一个模式的评分的正负技巧
    :param df_mid:
    :param method:
    :param reference_member:
    :param s:
    :param save_path:
    :param kwargs:
    :return:
    '''
    grd = score_xz_df(df_mid,method,s = s,g = "member",gll_dict=gll_dict,plot = False)
    grd_delta = copy.deepcopy(grd)
    member = grd["member"].values
    for i in range(member.size):
        if reference_member==member[i]:
            grd_delta.values[:] -= grd.values[i,:]

    meteva.base.contourf_xz(grd_delta, save_path=save_path,cmap = cmap,sup_title=sup_title,**kwargs)
    return grd_delta

def score_yz_df(df_mid, method, s=None, g=None, gll_dict=None, save_path=None,plot = True,sup_title = "", **kwargs):
    g_list = []
    if isinstance(g, list):
        if len(g) > 1:
            print("score_xz_df函数暂时不支持多维分类功能")
            return
        else:
            g_list = g
    else:
        if g is not None:
            g_list = [g]

    g_list.append("lat")
    g_list.append("level")
    if s is not None:
        if "member" in s.keys():
            df_mid = df_mid.loc[df_mid['member'].isin(s["member"])]
            s.pop("member")
        if "grade" in s.keys():
            df_mid = df_mid.loc[df_mid['grade'] == s["grade"]]
            s.pop("grade")


    ids = df_mid["id"]  # 所有格点的编号
    max_id = np.max(ids)
    min_id = np.min(ids)
    max_ = max(max_id, abs(min_id))
    if max_ < 1000000:
        lat = np.round(ids / 1000)
        lon = ids - lat * 1000
    else:
        lat = np.round(ids / 10000)
        lon = ids - lat * 10000
        lon /= 10
        lat /= 10
    df_mid["lat"] = lat
    df_mid["lon"] = lon
    # print(df_mid)
    df_mid = meteva.base.sele_by_dict(df_mid, s)

    score_array, g_dict = score_df(df_mid, method, g=g_list, gll_dict=gll_dict)

    coords = []
    for g1 in g_dict.keys():
        coords.append((g1, g_dict[g1]))
    da = xr.DataArray(score_array, coords=coords)

    grd = meteva.base.xarray_to_griddata(da)
    if plot :meteva.base.contourf_yz(grd, save_path=save_path,sup_title=sup_title, **kwargs)
    return grd


def score_tdt_df(df_mid, method, s=None, g=None, gll_dict=None, save_path=None,sup_title = "", **kwargs):
    g_list = []
    if isinstance(g, list):
        if len(g) > 1:
            print("score_tdt_df函数暂时不支持多维分类功能")
            return
        else:
            g_list = g
    else:
        if g is not None:
            g_list = [g]

    g_list.append("dtime")
    g_list.append("time")

    result = score_df(df_mid,method,s = s,g= g_list,gll_dict=gll_dict,save_path = save_path,plot="mesh",sup_title=sup_title,**kwargs)
    return result


def score_tdt_df_delta(df_mid, method,reference_member, s=None, gll_dict=None,cmap ="me_bwr", **kwargs):

    g_list=["member","dtime","time"]
    score_array,gll_dict = score_df(df_mid,method,s = s,g= g_list,gll_dict=gll_dict)
    member = gll_dict["member"]
    for i in range(len(member)):
        if reference_member == member[i]:
            score_array[:] -= score_array[i, :]
    meteva.base.plot_tools.mesh_contourf("mesh", score_array, name_list_dict=gll_dict,cmap=cmap, **kwargs)
    return score_array,gll_dict

def score_yz_df_delta(df_mid,method,reference_member,s = None,gll_dict = None,save_path = None,cmap ="me_bwr",sup_title = "",**kwargs):
    '''
    绘制多个模式预报评分相对其中某一个模式的评分的正负技巧
    :param df_mid:
    :param method:
    :param reference_member:
    :param s:
    :param save_path:
    :param kwargs:
    :return:
    '''
    grd = score_yz_df(df_mid,method,s = s,g = "member",gll_dict=gll_dict,plot = False)
    grd_delta = copy.deepcopy(grd)
    member = grd["member"].values
    for i in range(member.size):
        if reference_member==member[i]:
            grd_delta.values[:] -= grd.values[i,:]
    meteva.base.contourf_yz(grd_delta, save_path=save_path,cmap = cmap,sup_title=sup_title,**kwargs)
    return grd_delta


if __name__ =="__main__":
    #
    path = r"H:\task\other\202308-AImodel\mid\z500_tmmsss_ec.h5"
    df = pd.read_hdf(path)

    score_xy_df_delta(df,method=meteva.method.ob_fo_std,reference_member="OBS",save_path=r"H:\task\other\202308-AImodel\mid\me_test_.png",
                     s = {"member":["ECMWF","CMA_GFS"]},
                      gll_dict={"member":["OBS","CMA_GFS","ECMWF"]})



import meteva
import numpy as np
from meteva.base import IV
from meteva.method.yes_or_no.score import ts_hfmc, ets_hfmc, bias_hfmc, far_hfmc, mr_hfmc,pod_hfmc,sr_hfmc,pofd_hfmc
import math


def tc(ob, fo, grade_list=None):
    '''
    accuracy 求多分类预报准确率
    :param ob: 实况数据 不确定维numpy
    :param fo:  预测数据 不确定维numpy
    :param grade_list:等级，如果grade_list= None则ob和fo里的值代表类别，否则，根据grade_list来进行分类
    :return: 返回一维数组，包括（总样本数，正确数）
    '''
    tc_array_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    for line in range(new_Fo_shape[0]):
        # 多分类预报准确率
        if grade_list is None:
            # ob 和fo是不确定维的numpy数组，其中每个值是代表分类的类别
            ob1 = ob.reshape((-1))
            fo1 = new_Fo[line, :].reshape((-1))
        else:
            ob1 = np.zeros_like(ob)
            fo1 = np.zeros_like(new_Fo[line, :])
            # ob 和fo 是连续的变量，通过 threshold_list 将ob 和fo划分成连续的等级之后再计算等级准确性
            for index in range(len(grade_list) - 1):
                ob_index_list = np.where((grade_list[index] <= ob) & (ob < grade_list[index + 1]))
                ob1[ob_index_list] = index + 1
                fo_index_list = np.where(
                    (grade_list[index] <= new_Fo[line, :]) & (new_Fo[line, :] < grade_list[index + 1]))
                fo1[fo_index_list] = index + 1
            ob_index_list = np.where(grade_list[-1] <= ob)
            ob1[ob_index_list] = len(grade_list)
            fo_index_list = np.where(grade_list[-1] <= new_Fo[line, :])
            fo1[fo_index_list] = len(grade_list)
        correct_num = np.sum(fo1 == ob1)

        tc_array_list.append(np.array([ob.size, correct_num]))
    tc_array = np.array(tc_array_list)
    shape = list(Fo_shape[:ind])
    shape.append(2)
    tc_array = tc_array.reshape(shape)
    return tc_array


def tcof(ob, fo, grade_list=None):
    '''
    多分类评分中间统计量
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 一个从小到大排列的实数列表，以其中列出的数值划分出的多个区间作为分类标签。
    :return: 一维数组，包括（总样本数，正确样本数，观测的样本数、预报的样本数）
    '''
    tcof_list = []
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])

    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    if (grade_list is None):
        fo_list = list(set(fo.reshape((-1)).tolist()))
        ob_list = list(set(fo.reshape((-1)).tolist()))
        fo_list.extend(ob_list)
        grade_list = list(set(fo_list))
        grade_list.sort()
        if len(grade_list) > 30:
            print("自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省")
            return
    for line in range(new_Fo_shape[0]):
        tc1 = tc(ob, new_Fo[line, :],grade_list=grade_list)
        ft = meteva.method.multi_category.table.frequency_table(ob, new_Fo[line, :], grade_list=grade_list)
        tcof1 = np.zeros((len(grade_list) + 2, 2))
        tcof1[0, :] = tc1
        tcof1[1:, :] = ft.T
        tcof_list.append(tcof1)

    tcof_array = np.array(tcof_list)

    shape = list(Fo_shape[:ind])
    shape_list = [len(grade_list) + 2, 2]
    shape.extend(shape_list)
    tcof_array = tcof_array.reshape(shape)

    return tcof_array


def hfmc_grade(ob, fo, grade_list):
    '''
    多分类评分中间统计量
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 一个从小到大排列的实数列表，以其中列出的数值划分出的多个区间作为分类标签。
    :return: 一维数组，包括（总样本数，正确样本数，观测的样本数、预报的样本数）
    '''
    hfmc_multi_list = []
    Ob_shape = ob.shape
    Fo_shape = fo.shape

    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return

    total_count = ob.size
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    new_grade_list = None
    for line in range(new_Fo_shape[0]):

        new_grade_list = grade_list
        hfmc_m = np.zeros((len(grade_list) + 1, 4))
        gle = [-1e300]
        gle.extend(grade_list)
        gle.append(1e300)
        for i in range(len(gle) - 1):
            #命中样本
            hit_index_list = np.where(
                (ob >= gle[i]) & (ob < gle[i + 1]) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
            hfmc_m[i, 0] = len(hit_index_list[0])

            #空报样本，预报处于某个区间，而观测低于该区间
            fal_index_list = np.where(
                (ob < gle[i]) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
            hfmc_m[i, 1] = len(fal_index_list[0])

            # 漏报样本，观测处于某个区间，而预测低于该区间
            mis_index_list = np.where(
                (ob >= gle[i]) & (ob < gle[i + 1]) & (new_Fo[line, :] < gle[i]))
            hfmc_m[i, 2] = len(mis_index_list[0])

            #正确无
            nd_list = np.where((ob < gle[i]) & (new_Fo[line, :] < gle[i]))
            hfmc_m[i, 3] = len(nd_list[0])
        hfmc_multi_list.append(hfmc_m)
    hfmc_multi_array = np.array(hfmc_multi_list)
    shape = list(Fo_shape[:ind])
    a = len(new_grade_list)
    if (grade_list is not None):
        a+=1
    shape.append(a)
    shape.append(4)
    hfmc_multi_array = hfmc_multi_array.reshape(shape)
    return hfmc_multi_array


def hfmc_multi(ob, fo, grade_list=None):
    '''
    多分类评分中间统计量
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 一个从小到大排列的实数列表，以其中列出的数值划分出的多个区间作为分类标签。
    :return: 一维数组，包括（总样本数，正确样本数，观测的样本数、预报的样本数）
    '''
    hfmc_multi_list = []
    Ob_shape = ob.shape
    Fo_shape = fo.shape

    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return

    total_count = ob.size
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape
    new_grade_list = None
    for line in range(new_Fo_shape[0]):

        if (grade_list is None):
            grade_list1 = list(set(ob.flatten()))
            grade_list2 = list(set(fo.flatten()))
            grade_list1.extend(grade_list2)
            new_grade_list = list(set(grade_list1))
            new_grade_list.sort()
            if len(new_grade_list) > 30:
                print("自动识别的样本类别超过30种，判断样本为连续型变量，grade_list不能缺省")
                return
            hfmc_m = np.zeros((len(new_grade_list), 4))
            for i in range(len(new_grade_list)):
                grade = new_grade_list[i]
                hit_index_list = np.where((ob == grade) & (new_Fo[line, :] == grade))

                hfmc_m[i, 0] = len(hit_index_list[0])
                fal_index_list = np.where((ob != grade) & (new_Fo[line, :] == grade))
                hfmc_m[i, 1] = len(fal_index_list[0])
                mis_index_list = np.where((ob == grade) & (new_Fo[line, :] != grade))
                hfmc_m[i, 2] = len(mis_index_list[0])
                hfmc_m[i, 3] = total_count - hfmc_m[i, 0] - hfmc_m[i, 1] - hfmc_m[i, 2]
        else:
            new_grade_list = grade_list
            hfmc_m = np.zeros((len(grade_list) + 1, 4))
            gle = [-1e300]
            gle.extend(grade_list)
            gle.append(1e300)
            for i in range(len(gle) - 1):

                #命中样本
                hit_index_list = np.where(
                    (ob >= gle[i]) & (ob < gle[i + 1]) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
                hfmc_m[i, 0] = len(hit_index_list[0])

                #空报样本，预报处于某个区间，而观测低于该区间
                fal_index_list = np.where(((ob < gle[i]) | (ob >= gle[i + 1])) & (new_Fo[line, :] >= gle[i]) & (new_Fo[line, :] < gle[i + 1]))
                hfmc_m[i, 1] = len(fal_index_list[0])
                # 漏报样本，观测处于某个区间，而预测低于该区间
                mis_index_list = np.where((ob >= gle[i]) & (ob < gle[i + 1]) & ((new_Fo[line, :] < gle[i]) | (new_Fo[line, :] >= gle[i + 1])))
                hfmc_m[i, 2] = len(mis_index_list[0])

                #正确无
                hfmc_m[i, 3] = total_count - hfmc_m[i, 0] - hfmc_m[i, 1] - hfmc_m[i, 2]
        hfmc_multi_list.append(hfmc_m)
    hfmc_multi_array = np.array(hfmc_multi_list)
    shape = list(Fo_shape[:ind])
    a = len(new_grade_list)
    if (grade_list is not None):
        a+=1
    shape.append(a)
    shape.append(4)
    hfmc_multi_array = hfmc_multi_array.reshape(shape)

    return hfmc_multi_array


def accuracy(ob, fo, grade_list=None):
    '''
    accuracy 求多分类预报准确率
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :return: 0-1的实数，0代表无技巧，最优预报为1
    '''
    tc1 = tc(ob,fo,grade_list)
    total_count = tc1[...,0]
    correct_count = tc1[...,1]
    accuracy_score = correct_count / total_count
    return accuracy_score


def accuracy_tc(tc_array):
    '''
    :param tc_array:
    :return:
    '''
    total_count = tc_array[..., 0]
    correct_count = tc_array[..., 1]
    accuracy_score = correct_count / total_count
    return accuracy_score


def accuracy_tcof(tcof_array):
    '''

    :param tcof_array:
    :return:
    '''
    total_count = tcof_array[..., 0, 0]
    correct_count = tcof_array[..., 0, 1]
    accuracy_score = correct_count / total_count
    return accuracy_score


def ts_grade(ob,fo,grade_list):
    '''
    分级ts评分
    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    ts_array = ts_hfmc(hfmc_array)
    return ts_array


def ets_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    ets_array = ets_hfmc(hfmc_array)
    return ets_array


def bias_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    bias_array = bias_hfmc(hfmc_array)
    return bias_array


def mr_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    mr_array = mr_hfmc(hfmc_array)
    return mr_array


def far_grade(ob, fo, grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    far_array = far_hfmc(hfmc_array)
    return far_array

def pod_grade(ob,fo,grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    pod_array = pod_hfmc(hfmc_array)
    return pod_array

def pofd_grade(ob,fo,grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    pofd_array = pofd_hfmc(hfmc_array)
    return pofd_array

def sr_grade(ob,fo,grade_list):
    hfmc_array = hfmc_grade(ob, fo, grade_list)
    sr_array = sr_hfmc(hfmc_array)
    return sr_array


def ts_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    ts_array = ts_hfmc(hfmc_array)
    return ts_array


def ets_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    ets_array = ets_hfmc(hfmc_array)
    return ets_array


def bias_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    bias_array = bias_hfmc(hfmc_array)
    return bias_array


def mr_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    mr_array = mr_hfmc(hfmc_array)
    return mr_array


def far_multi(ob, fo, grade_list=None):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    far_array = far_hfmc(hfmc_array)
    return far_array


def pod_multi(ob,fo,grade_list):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    pod_array = pod_hfmc(hfmc_array)
    return pod_array

def pofd_multi(ob,fo,grade_list):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    pofd_array = pofd_hfmc(hfmc_array)
    return pofd_array

def sr_multi(ob,fo,grade_list):
    hfmc_array = hfmc_multi(ob, fo, grade_list)
    sr_array = sr_hfmc(hfmc_array)
    return sr_array


def hss(ob, fo, grade_list=None):
    '''
    hss heidke技能得分,它表现实际的预报的分类准确性相对于随机分类达到的准确性的技巧
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :return:
    '''
    Ob_shape = ob.shape
    Fo_shape = fo.shape
    hss_list = []
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')

        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    for line in range(new_Fo_shape[0]):

        conf_mx = meteva.method.multi_category.table.contingency_table_multicategory(ob, new_Fo[line, :], grade_list)

        accuracy_score = accuracy(ob, new_Fo[line, :], grade_list)
        total_num = ob.size
        NF_array = conf_mx[0:-1, -1]
        NO_array = conf_mx[-1, 0:-1]
        random_score = np.dot(NF_array, NO_array) / (total_num * total_num)
        if (random_score == 1):
            HSS = IV
        else:
            HSS = (accuracy_score - random_score) / (1 - random_score)
        hss_list.append(HSS)
    if len(hss_list)==1:
        hss_array = hss_list[0]
    else:
        hss_array = np.array(hss_list)
        shape = list(Fo_shape[:ind])
        hss_array = hss_array.reshape(shape)

    return hss_array


def hk(ob, fo, grade_list=None):
    '''
    hk Hanssen和Kuipers判别
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :return:
    '''
    # 多分类预报hk技巧评分
    Ob_shape = ob.shape
    Fo_shape = fo.shape
    hk_list = []
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')

        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape

    for line in range(new_Fo_shape[0]):
        conf_mx = meteva.method.multi_category.table.contingency_table_multicategory(ob, new_Fo[line, :], grade_list)
        accuracy_score = accuracy(ob, new_Fo[line, :], grade_list)
        total_num = ob.size
        NF_array = conf_mx[0:-1, -1]
        NO_array = conf_mx[-1, 0:-1]
        random_score = np.dot(NF_array, NO_array) / (total_num * total_num)
        ob_rate = np.dot(NO_array, NO_array) / (total_num * total_num)
        HK = (accuracy_score - random_score) / (1 - ob_rate)
        hk_list.append(HK)
    if len(hk_list)==1:
        hk_array = hk_list[0]
    else:
        hk_array = np.array(hk_list)
        shape = list(Fo_shape[:ind])
        hk_array = hk_array.reshape(shape)

    return hk_array


def hk_tcof(tcof_array):
    '''
    :param tcof_array:
    :return:
    '''

    tc_sum = tcof_array[..., 0, 0]
    cc_sum = tcof_array[..., 0, 1]
    accuracy_score = cc_sum / tc_sum
    obn_sum = tcof_array[..., 1:, 0]
    fon_sum = tcof_array[..., 1:, 1]
    dotobfo = np.zeros_like(tc_sum)
    dotobob = np.zeros_like(tc_sum)
    for g in range(obn_sum.shape[-1]):
        dotobfo += obn_sum[..., g] * fon_sum[..., g]
        dotobob += obn_sum[..., g] * obn_sum[..., g]
    random_score = dotobfo / (tc_sum * tc_sum)
    ob_rate = dotobob / (tc_sum * tc_sum)
    if ob_rate.size == 1:
        if ob_rate == 1:
            HK = IV
        else:
            HK = (accuracy_score - random_score) / (1 - ob_rate)
    else:
        ob_rate[ob_rate == 1] = -1
        HK = (accuracy_score - random_score) / (1 - ob_rate)
        HK[ob_rate == -1] = IV
    return HK


def hss_tcof(tcof_array):
    '''
    :param tcof_list:
    :return:
    '''
    tc_sum = tcof_array[..., 0, 0]
    cc_sum = tcof_array[..., 0, 1]
    accuracy_score = cc_sum / tc_sum
    obn_sum = tcof_array[..., 1:, 0]
    fon_sum = tcof_array[..., 1:, 1]
    dotobfo = np.zeros_like(tc_sum)
    dotobob = np.zeros_like(tc_sum)
    for g in range(obn_sum.shape[-1]):
        dotobfo += obn_sum[..., g] * fon_sum[..., g]
        dotobob += obn_sum[..., g] * obn_sum[..., g]
    random_score = dotobfo / (tc_sum * tc_sum)

    if random_score.size == 1:
        if random_score == 1:
            HSS = IV
        else:
            HSS = (accuracy_score - random_score) / (1 - random_score)
    else:
        random_score[random_score == 1] = -1
        HSS = (accuracy_score - random_score) / (1 - random_score)
        HSS[random_score == -1] = IV

    return HSS


def seeps_ctable(contingency_table_multicategory,p1):

    s_fv = np.array([[0, 1 / (1 - p1), 4 / (1 - p1)],
                     [1 / p1, 0, 3 / (1 - p1)],
                     [1 / p1 + 3 / (2 + p1), 3 / (2 + p1), 0]]) * 0.5

    if len(contingency_table_multicategory.shape) == 2:
        ctm_fv = contingency_table_multicategory
        p_fv = ctm_fv[0:-1, 0:-1] / ctm_fv[-1, -1]
        #print(p_fv)
        seeps_error = np.sum(s_fv * p_fv)
        return seeps_error
    else:
        seeps_error_list=[]
        for line in range(contingency_table_multicategory[0]):
            ctm_fv = contingency_table_multicategory[line,:,:]
            p_fv = ctm_fv[0:-1, 0:-1] / ctm_fv[-1, -1]
            seeps_error = np.sum(s_fv * p_fv)
            seeps_error_list.append(seeps_error)
        seeps_error_array = np.array(seeps_error_list)
        #shape = list(Fo_shape[:ind])
        #seeps_error_array = seeps_error_array.reshape(shape)
        return seeps_error_array


def seeps(Ob, Fo, p1=None, threshold=None):
    '''

    :param Ob:
    :param Fo:
    :param p1:
    :param significant_rain_threshold:
    :return:
    '''

    if p1 is None:
        ob1 = np.sort(Ob.flatten())
        index_p1 = np.where(ob1 <= 0.2)
        p1 = len(index_p1[0]) / ob1.size
        p3 = (1 - p1) / 3
        index_p3 = int(math.ceil((1 - p3) * ob1.size)) - 1
        threshold = ob1[index_p3]
        if threshold < 0.3:
            threshold = 0.3
    if p1 == 1: p1 -= 1e-6
    if p1 == 0: p1 = 1e-6
    ctable = meteva.method.contingency_table_multicategory(Ob, Fo,
                                                  grade_list=[0.2000001, threshold])
    seeps_error = seeps_ctable(ctable,p1)
    return seeps_error



def seeps_skill(Ob, Fo, p1=None, threshold=None):
    '''

    :param Ob:
    :param Fo:
    :param p1:
    :param significant_rain_threshold:
    :return:
    '''
    seeps_array = seeps(Ob,Fo,p1= p1,threshold=threshold)
    skill = 1- seeps_array
    return skill


def show_valid_k1_k2(p1,p2,p3):
    x = np.arange(201)*0.01-1
    y = np.arange(201)*0.01-1
    valid_index = np.zeros((201,201))
    for i in range(201):
        for j in range(201):
            k2 = x[i]
            k1 = y[j]

            s11 = (p3 + p1*(p3-p2) * k1 + p3*(p2+p3)*k2) / (p1 * (p1+p3))
            s13 = -(1+(p1+p2)*k1 + (p2+p3)*k2)/(p1+p3)
            s22 = -(p1*k1 + p3 *k2)/p2
            s33 = (p1+p1*(p1+p2)*k1 + p3*(p1-p2)*k2)/(p3*(p1+p3))

            if (i == 125 and j == 50):
                print()

            # 非顺序条件
            if k1 < s11 and k1 < s22 and k2 <s22 and k2 < s33 and s13 < s11 and s13 < s33:
                valid_index[i,j] = 1

                #顺序条件
                if s13< k1 and s13 < k2:
                    valid_index[i, j] += 1
    print(valid_index)
    import matplotlib.pyplot as plt
    plt.pcolormesh(valid_index.T)
    plt.show()

def get_s_array(p1,p2,p3,k1=-0.25,k2=-0.25):
    '''
    根据3分类预报问题的实况气候概率p1,p2和p3， 确定Murphy评分的评分权重矩阵。
    评分权重的确定原理简单叙述如下：
    3分类预报的完整检验信息可由一个3×3的列联表表示。列联表中的元素p_ij表示 观测为第i类,预报为第j类的样本数。
    总的评分等于列联表每个元素×一个权重系数后求和得到 sum(p_ij * s_ij)。 为此需要确定9个评分权重系数。
    一般来说对角线上的权重系数s_ii 应该是正的，表示奖励， 非对角线上的s_ij是负的表示惩罚。那这9个评分权重该取为多少，
    用它们做评价才是公平的？ 为此主要考虑2点，一是如果全部报对（列联表的元素全部集中在对角线上），评分应为1，另外，
    随机预报的评分应该为0，这2个条件可以构成关于权重系数的4个方程构成的方程组，这个方程组中包含3个分类的实况概率p1、p2、p3，
    构成的已知参数。如果考虑这个评分矩阵是对称的，则一共有6个待定系数，因此它是一个欠定问题。为此有两个评分权重系数必须事先指定，
    为此令 k1 = s12, k2 = s23， 在此基础上可以计算出其它4个系数 s11,s22,s33,s13.
    根据Murphy的分析，k1和k2的取值范围为(-0.5,0)时，计算出的评分权重可以满足：
    s12 < s11,s12<s22, s23<s22,s23<s33, s13<s12,s13<s23。
    关于上述内容的更详细论述可参考《预报检验-大气科学从业者指南》 4.3.2节
    :param p1: 实况第1分类的概率
    :param p2: 实况第2分类的概率
    :param p3: 实况第3分类的概率
    :param k1:权重系数s12
    :param k2:权重系数s23
    :return: 3×3的权重系数矩阵。
    '''

    s_array = np.zeros((3,3))

    s_array[0,1] = k1
    s_array[1, 0] = k1  #对称
    s_array[1,2] = k2
    s_array[2, 1] = k2  #对称

    #《预报检验-大气科学从业者指南》 4.3.2节 公式4.13
    s_array[0,0] = (p3 + p1 * (p3 - p2) * k1 + p3 * (p2 + p3) * k2) / (p1 * (p1 + p3))
    s_array[0, 2] = -(1 + (p1 + p2) * k1 + (p2 + p3) * k2) / (p1 + p3)
    s_array[1,1] = -(p1 * k1 + p3 * k2) / p2
    s_array[2,2] = (p1 + p1 * (p1 + p2) * k1 + p3 * (p1 - p2) * k2) / (p3 * (p1 + p3))

    s_array[2,0] = s_array[0,2]  #对称

    return s_array



def murphy_ctable(p_array,s_array):
    '''
    根据3×3的列联表，和评分权重矩阵 计算 Murphy 矩阵
    评分公式参考《预报检验-大气科学从业者指南》 4.3.2节，公式 4.8
    :param ctable_array:  3×3的观测预报列联表
    :param s_array:  评分权重矩阵
    :return:  Murphy评分
    '''

    score = np.sum(p_array * s_array)
    return score



def murphy_score(ob,fo,grade_list = None,p1 = None,p2 = None,p3 = None,k1 = -0.25,k2 = 0.25):
    '''
    计算Murphy 评分。
    步骤包括：
    1. 根据实况和预报数据，以及等级划分阈值，统计列联表
    2. 根据3分类预报问题的实况气候概率p1,p2和p3，确定Murphy评分的评分权重矩阵（函数 get_s_array）
    3. 根据列联表和权重矩阵，计算Murphy 评分。

    :param ob: 实况数据，
    :param fo: 预报数据
    :param grade_list:实况和预报分类的阈值，如果实况和预报已经是离散的等级，则不需要该参数
    :param p1: 实况为第1分类的气候概率
    :param p2:实况为第2分类的气候概率
    :param p3:实况为第2分类的气候概率
    :param k1: 评分权重系数 s12
    :param k2:评分权重系数 s23
    :return: Murphy评分。
    '''

    #根据实况获取观测预报的列联表
    ctable = meteva.method.multi_category.table.contingency_table_multicategory(ob,fo,grade_list)
    if len(ctable.shape)==2:
        ctable = ctable.reshape(1,ctable.shape[0],ctable.shape[1])

    ctable = ctable/ctable[-1,-1]
    if p1 is not None and p2 is not None and p3 is not None:
        # 如果用户从外部指定实况的概率，就用用户指定的概率来生成权重矩阵s_array
        s_array = get_s_array(p1,p2,p3,k1,k2)
    else:
        #如果用户未指定实况概率，则根据传入的实况数据来获取每个等级的概率，再计算权重矩阵
        p_ob = ctable[0, :-1, -1] / ctable[0, -1, -1]  # 返回的列联表实际上是4×4的，其中最后一行是实况的概率分布
        s_array = get_s_array(p_ob[0], p_ob[1], p_ob[2], k1, k2)


    score_list = []
    for i in range(ctable.shape[0]):
        ct_array = ctable[i, :-1, :-1]
        score1 =murphy_ctable(ct_array,s_array)  #已知列联表和权重系数矩阵，计算Murphy 评分
        score_list.append(score1)

    if len(score_list)==1:
        return score_list[0]  #单个预报返回实数
    else:
        return np.array(score_list)  #多个预报返回数组





if __name__ == "__main__":


    s_array = get_s_array(-0.5,-0.25,0.2,0.5,0.3)
    print(s_array*60)


# -*- coding: utf-8 -*-
"""
Created on Tue Jun  9 11:40:04 2020

@author: 1
"""

#======================  make.SpatialVx()  =======================

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import sys
import math

def isAllDigital(array):      #这个函数是用来判断list是否为一维纯数值，如果是二维，返回false
    ret = True
    for v in array:
        ret = ret and isinstance(v,(int,float,np.int,np.int32))
        if not ret:
            return False
    return ret


def make_spatialVx(grd_ob, grd_fo, loc=None,
                   fieldtype = "", units = "", dataname = "",
                   obsname = "X", modelname = "Xhat", thresholds=None):
    #参数：X, Xhat, threshold, loc, projection, subset, timevals,reggrid, map, locbyrow, fieldtype, units, dataname, obsname, modelname, q, qs
    if thresholds is None:
        thresholds = [None, None]
    X = grd_ob
    Xhat = grd_fo
    projection = False
    subset = None
    timevals = None
    reggrid = True
    Map = False
    locbyrow = False
    qs = None
    q = (0.05, 0.1, 0.25, 0.333, 0.5, 0.666, 0.75, 0.9, 0.95)

    #if (X.shape == None):
    if (type(X) == list):    #判断X是否为列表，X实际为二维矩阵
        nobs = X.shape[0] * X.shape[1]
        xdim = None
    else:
        nobs = 1
        xdim = X.shape

    #if (Xhat.shape == None):
    if (type(Xhat) == list):    #判断X是否为列表
        nforecast = Xhat.shape[0] * Xhat.shape[1]
        ydim = None
    else:
        nforecast = 1
        ydim = Xhat.shape


    if (xdim != ydim):
        try:
            sys.exit(0)
        except:
            print("make.SpatialVx: dim of X{0}  must be the same as dim of Xhat{1} ".format(X.shape,Xhat.shape))

    out = {"X": X, "Xhat": Xhat}    #将矩阵存为列表


    if thresholds == [None, None]:
        if (nobs > 1):    #猜测是输入的观测数据大于一个时次，对每个时次的数据分别求分位数
            othresh = np.zeros((9,1))    #定义othresh为空数组，长度与q一致，9
            for i in range(1, nobs):
                #X_1 = np.array(X)
                #X_flat = X_1.flatten()    #dataframe没有flatten属性，先转array
                quantile_X = np.quantile(X, q)  #计算X的分位数
                othresh = np.hstack([quantile_X, othresh])
        else:
            othresh = np.quantile(X, q)    #数据大小为（9，1）是写死的，因为所要求的的分位数的个数是确定的

        if (nforecast > 1):    #猜测是输入的观测数据大于一个时次，对每个时次的数据分别求分位数
            fthresh = np.zeros()    #定义fthresh为空数组，长度未知
            for i in range(1, nforecast):
                quantile_Xhat = np.quantile(Xhat, q)  #计算Xhat的q分位数
                fthresh = np.hstack([fthresh, quantile_Xhat])  #原函数里面直接生成的矩阵，没有赋值到任何变量
        else:
            fthresh = np.quantile(Xhat, q)    #数据大小为（9，1）是写死的，因为所要求的的分位数的个数是确定的
        qs = str(q)
        #X = othresh
        #Xhat = fthresh
        thresholds = [othresh, fthresh]

    elif isinstance(thresholds, pd.DataFrame):    #判断thresholds是否为一个list
        threshnames = ["X","Xhat"]    #获取列表的名称
        if (("X"not in threshnames) or ("Xhat" not in threshnames)):
            try:
                sys.exit(0)
            except:
                print("make.SpatialVx: invalid thresholds argument.  List must have components named X and Xhat.")

        odim = np.array([len(thresholds), np.ndim(thresholds)])    #thresholds 第一项(X)的数组长度赋值给odim
        fdim = np.array([len(thresholds), np.ndim(thresholds)])    #thresholds 第二项(Xhat)的数组长度赋值给fdim


        if(odim[0] != fdim[0]):
            try:
                sys.exit(0)
            except:
                print("make.SpatialVx: invalid thresholds argument.  X must have same number of thresholds (rows) as Xhat.")
        nth = odim[0]

        if(any(odim) == None):
            thresholds[0] = np.array(thresholds[0]).reshape(len(thresholds[0]),nobs)
        elif(odim[1] == 1 and nobs > 1):
            thresholds[0] = np.array(thresholds[0]).reshape(len(thresholds[0]),nobs)
        elif (odim[1] > 1 and odim[1] != nobs):
            try:
                sys.exit(0)
            except:
                print("make.SpatialVx: invalid thresholds argument.")
        threshold = 'threshold'
        qs = []
        for i in range(nth):
            qs.append(threshold +' '+ str(i))


    elif(type(thresholds) == list):    #判断thresholds是否为vector,R里面检查是否最多只有一个属性：name。即查看其属性，如果没有属性或者只有一个name属性，才返回TRUE

        qs = str(thresholds)
        nth = len(thresholds)
        thresholds = [np.array(thresholds).reshape(nth,nobs),np.array(thresholds).reshape(nth,nforecast)]
        #else stop("make.SpatialVx: invalid thresholds argument.  Must be a vector or list.")
    else:
        try:
            sys.exit(0)
        except:
            print("make.SpatialVx: invalid thresholds argument.  Must be a vector or list.")

    #if (loc.any() is None):    #判断loc是否为空,如果为空的话，生成坐标信息
    if (loc is None):  # 判断loc是否为空,如果为空的话，生成坐标信息
        rep00_1 = np.arange(0, xdim[0], 1)
        rep00 = np.tile(rep00_1, xdim[1])
        rep0_1 = np.arange(0, xdim[1], 1)
        rep0 = rep0_1.repeat(xdim[0], axis = 0)    #按行进行元素重复
        loc = np.vstack((rep00, rep0))
    if (timevals == None):
        if (len(xdim) == 3):
            timevals = np.arnge(1, xdim[2])
        else:
            timevals = 1
    if (dataname != None):
        msg = dataname
    else :
        msg = ""
    if (fieldtype != "" and units != ""):
        msg = msg + '\n' + fieldtype +' ' + '(' + units + ')'
    elif (fieldtype != ""):
        msg = msg + '\n' + fieldtype
    elif (units != ""):
        msg = msg + ' ' + '(' + units + ')'


    out = {"X": X, "Xhat": Xhat, "class":"SpatialVx", "xdim":xdim, "time":timevals,
               "thresholds":thresholds, "loc":loc, "locbyrow":locbyrow,
               "subset":subset, "dataname":dataname, "obsname":obsname,
               "modelname":modelname, "nobs":nobs, "nforecast":nforecast,
               "fieldtype":fieldtype , "units":units , "projection":projection ,
               "reggrid":reggrid , "Map":Map , "qs":qs, "msg":msg}
    '''
    #数据
    out = {"grd_ob": X, "grd_fo": Xhat, "class":"SpatialVx", "xdim":xdim, "loc":loc,
          "dataname":dataname, "obsname":obsname, "modelname":modelname, 
          "fieldtype":fieldtype, "units":units ,"msg":msg}    
    '''
    return out

'''
          
if __name__ == '__main__':
hold = make_spatialVx(grd_ob = pd.read_csv(r'G:\Work\MODE\tra_test\FeatureFinder\pert000.csv'),
                     grd_fo = pd.read_csv(r'G:\Work\MODE\tra_test\FeatureFinder\pert004.csv'), 
                     loc = pd.read_csv(r'G:\\Work\\MODE\\tra_test\\FeatureFinder\\ICPg240Locs.csv'),
                     fieldtype = "Precipitation")
'''





















import meteva
import pandas as pd
import numpy as np
import math
import datetime


'''
#该模块将逐步包含业务规范中的短时强降水的检验规则
'''


def edition_2020_1_lightning_ob_to_count(lightning_ob, station, dlon_dlat = 0.25):
    '''
    该模块实现的功能是先将不规则时间、空间发生的闪电，通过最近距离法记录到规则网格的闪电次数中。
    再采用临近点插值方法将规则网格的闪电次数插值到国家站和区域站

    :param lightning_ob:   sta_data 形式的闪电观测数据，它是一个DataFrame， 包含7列内容，分别为 level, time,dtime,id,lon,lat, 闪电强度。
                            其中time,lon,lat 是计算所需的内容，  level， dtime,id，闪电强度等列的内容可以为任意值。
    :param station:    网格插值到站点用的站点列表， sta_data 数据格式，其中数据列名称为type, 国家站的取值为1，其它站为2
    :param dlon_dlat ： 闪电次数网格化时用的网格距，根据2020年版本的检验方案，该该参数缺省值取为0.25
    :return:  站点形式的闪电次数，数据格式为sta_data
    '''
    # 根据原始的闪电数据获取逐小时记录的闪电数据
    # 如果lightning_ob已经是逐小时的数据了，下面一行语句也可以正常工作不会导致的错误，但比直接赋值的方式计算略多
    sta_ob_lightning_hourly = meteva.base.time_ceilling(lightning_ob, 1)

    # 根据闪电数据获取网格化的闪电次数
    #根据站点数据设置能够包含所有站点的网格范围
    min_lon1 = min(lightning_ob["lon"])
    min_lon2 = min(station["lon"])
    min_lon = min(min_lon1,min_lon2)
    min_lon = math.floor(min_lon / dlon_dlat) * dlon_dlat
    max_lon1 = max(lightning_ob["lon"])
    max_lon2 = max(station["lon"])
    max_lon = max(max_lon1,max_lon2)
    max_lon = math.ceil(max_lon / dlon_dlat) * dlon_dlat
    min_lat1 = min(lightning_ob["lat"])
    min_lat2 = min(station["lat"])
    min_lat = min(min_lat1, min_lat2)
    min_lat = math.floor(min_lat / dlon_dlat) * dlon_dlat
    max_lat1 = max(lightning_ob["lat"])
    max_lat2 = max(station["lat"])
    max_lat = max(max_lat1, max_lat2)
    max_lat = math.ceil(max_lat / dlon_dlat) * dlon_dlat

    # 创建一个网格变量
    grid = meteva.base.grid([min_lon, max_lon, dlon_dlat], [min_lat, max_lat, dlon_dlat])


    # 将闪电数据分解一个小时一个DataFrame的形式，置于列表当中
    sta_ob_lightning_hourly_list = meteva.base.group(sta_ob_lightning_hourly, g="time")

    # 用于记录每个小时的闪电次数插值结果
    sta_lightning_count_list = []

    #循环对每个小时的数据进行转换
    for sta1 in sta_ob_lightning_hourly_list[0]:
        # 根据站点形式的闪电数据，统计出网格形式的闪电次数
        grid_lightning_count = meteva.base.add_stacount_to_nearest_grid(sta1, grid)

        # 将网格化闪电次数采用最邻近法插值到站点
        sta2 = meteva.base.interp_gs_nearest(grid_lightning_count, station)

        # 记录每小时的闪电次数插值结果。
        sta_lightning_count_list.append(sta2)

    # 将闪电次数观测数据拼接在一起。
    sta_ob_linghtning_count = pd.concat(sta_lightning_count_list, axis=0)

    return sta_ob_linghtning_count



def edition_2020_1_lightning_count_to_01(sta_ob_linghtning_count,station):
    '''
    该模块实现根据站点形式的闪电次数判断雷暴大风的第一个必要条件是否满足，满足就记为1，不满足就记为0.
    不同的站点类型可以设置不同的阈值条件
    :param sta_ob_linghtning_count:  sta_data形式的数据， 数据列为闪电次数，
    :param station:    网格插值到站点用的站点列表， sta_data 数据格式，其中数据列名称为type, 国家站的取值为1，其它站为2
    :return:  sta_data 形式数据。
    '''
    #创建一个站点形式的变量记录不同站点的阈值
    id_threshold_sta = station.copy()

    # 国家站阈值设置为1
    id_threshold_sta.loc[station["type"] ==1, "type"] = 1

    # 非国家站阈值设置4
    id_threshold_sta.loc[station["type"] !=1, "type"] = 4

    #不相干维度的坐标值设置为缺省值
    id_threshold_sta[["level","time","dtime"]] = meteva.base.IV

    #根据闪电判断雷暴大风的闪电观测必要条件
    sta_ob_01 = meteva.method.point_to_area.p2p_vto01(sta_ob_linghtning_count,threshold= id_threshold_sta)

    return  sta_ob_01


def edition_2020_1_light01_maxwind_to_ob01(sta_lightning_01,sta_ob_max_wind):
    '''
    根据站点形式的闪电次数达标情况， 和 站点形式的最大风速 判断单个站是否出现雷暴大风
    :param sta_lightning_01:   站点形式的闪电次数达标情况，根据lightning_cound_to_01 函数生成
    :param sta_ob_max_wind:   站点形式的最大风速（过去1小时内）
    :return:  站点形式数据，数据列为 各个站雷暴大风是否发生，1代表发生，0代表维发生
    '''
    #根据最大风速判断雷暴大风的风速观测必要条件
    sta_ob_c2 = meteva.method.point_to_area.p2p_vto01(sta_ob_max_wind,threshold= 17.2)

    #将闪电条件和风速条件相乘，获得雷暴大风的是否发生,站点数以sta_lightning_01为准，因为sta_lightning_01由检验站表station生成,缺省值记为999999
    sta_ob01 = meteva.base.mutiply_on_level_time_dtime_id(sta_lightning_01,sta_ob_c2,how="left",default=meteva.base.IV)

    return sta_ob01


def edition_2020_1_skill_caculation(sta_ob01,sta_fo_list,pcapital_fo_list):
    '''
    :param sta_ob: 观测降水站点数据合集,其中站点值为具体的降水量值
    :param sta_fo_list:列表，其中第0个元素为中央台的预报插值到站点得到的站点数据的合集，第1个元素为省台预报对应结果.
     其中的站点值为0或1,0代表预报短时强降水不发生，1代表预报短时强降水发生
    :param pcapital_fo_list:列表，其中第0个元素为中央台的预报插值到省会城市得到的站点数据的合集，第1个元素为省台省会预报.
     其中的站点值为0或1,0代表预报短时强降水不发生，1代表预报短时强降水发生
    :return:
    '''

    R = 40  # 邻域半径
    #根据邻域算法判断站点附近邻域范围内是否发生雷暴大风，0代表没发生，1代表发生了；
    # sta_ob 里的数据取值只有0和1， 所以取threshold = 0.5 可以区分单点是否发生雷暴大风
    sta_ob_neared = meteva.method.p2a_vto01(sta_ob01,r = R,threshold= 0.5)

    #在将观测和预报数据进行匹配对齐以及合并,其中观测值缺省的样本会被删除
    sta_all_01 = meteva.base.combine_on_obTime_id(sta_ob_neared,sta_fo_list,need_match_ob=True)

    #在将观测和省会预报数据进行匹配对齐以及合并,其中观测值缺省的样本会被删除
    sta_all_01_pcapital = meteva.base.combine_on_obTime_id(sta_ob_neared,pcapital_fo_list,need_match_ob=True)

    #按时效分类检验，所有站点
    pod_results = meteva.product.score(sta_all_01,meteva.method.pod,g = "dtime")[0]
    far_results = meteva.product.score(sta_all_01,meteva.method.far,g = "dtime")[0]

    #按时效分类检验，针对省会城市预报
    pod_results_pcapital = meteva.product.score(sta_all_01_pcapital,meteva.method.pod,g = "dtime")[0]
    far_results_pcapital = meteva.product.score(sta_all_01_pcapital,meteva.method.far,g = "dtime")[0]

    #计算预报技巧
    spo = np.zeros(12)  # 格点1小时命中率预报技巧(所有站点 )
    sfa = np.zeros(12)  # 格点1小时空报率预报技巧(所有站点 )
    spo_pcapital = np.zeros(12)   # 单站1小时命中率预报技巧(省会城市)
    sfa_pcapital = np.zeros(12)    # 单站1小时空报率预报技巧(省会城市)
    #循环12个时效
    for i in range(12):
        spo[i] = meteva.method.spo(pod_results[i, 1], pod_results[i, 0])  #命中率技巧
        sfa[i] = meteva.method.sfa(far_results[i, 1], far_results[i, 0])  #空报率技巧
        spo_pcapital[i] = meteva.method.spo(pod_results_pcapital[i, 1], pod_results_pcapital[i, 0])  #省会命中率技巧
        sfa_pcapital[i] = meteva.method.sfa(far_results_pcapital[i, 1], far_results_pcapital[i, 0])  #省会空报率技巧

    return spo,sfa,spo_pcapital,sfa_pcapital

def edtion_2020_1(lightning_ob,station,sta_ob_max_wind,sta_fo_list,pcapital_fo_list):
    '''
    将各模块串成一个调用整体。
    :param lightning_ob:   sta_data 形式的闪电观测数据，它是一个DataFrame， 包含7列内容，分别为 level, time,dtime,id,lon,lat, 闪电强度。
                            其中time,lon,lat 是计算所需的内容，  level， dtime,id，闪电强度等列的内容可以为任意值。
    :param station:    网格插值到站点用的站点列表， sta_data 数据格式，其中数据列名称为type, 国家站的取值为1，其它站为2
    :param sta_ob_max_wind:   站点形式的最大风速（过去1小时内）
    :param sta_fo_list:列表，其中第0个元素为中央台的预报插值到站点得到的站点数据的合集，第1个元素为省台预报对应结果.
     其中的站点值为0或1,0代表预报短时强降水不发生，1代表预报短时强降水发生
    :param pcapital_fo_list:列表，其中第0个元素为中央台的预报插值到省会城市得到的站点数据的合集，第1个元素为省台省会预报.
     其中的站点值为0或1,0代表预报短时强降水不发生，1代表预报短时强降水发生
    :return:
    '''
    sta_ob_linghtning_count = edition_2020_1_lightning_ob_to_count(lightning_ob,station)
    sta_lightning_01 = edition_2020_1_lightning_count_to_01(sta_ob_linghtning_count, station)
    sta_ob01 = edition_2020_1_light01_maxwind_to_ob01(sta_lightning_01,sta_ob_max_wind)
    #meteva.base.scatter_sta(sta_ob01)
    return edition_2020_1_skill_caculation(sta_ob01,sta_fo_list,pcapital_fo_list)


def edition_2021_1_lightning_ob_to_count(lightning_ob, station, dlon_dlat = 0.5):
    return edition_2020_1_lightning_ob_to_count(lightning_ob, station, dlon_dlat=dlon_dlat)


def edition_2021_1_lightning_count_to_01(sta_ob_linghtning_count,station):
    '''
    该模块实现根据站点形式的闪电次数判断雷暴大风的第一个必要条件是否满足，满足就记为1，不满足就记为0.
    不同的站点类型可以设置不同的阈值条件
    :param sta_ob_linghtning_count:  sta_data形式的数据， 数据列为闪电次数，
    :param station:    网格插值到站点用的站点列表， sta_data 数据格式，其中数据列名称为type, 国家站的取值为1，其它站为2
    :return:  sta_data 形式数据。
    '''
    #创建一个站点形式的变量记录不同站点的阈值
    id_threshold_sta = station.copy()

    # 国家站阈值设置为1
    id_threshold_sta.loc[station["type"] ==1, "type"] = 1

    # 非国家站阈值设置4
    id_threshold_sta.loc[station["type"] !=1, "type"] = 1

    #不相干维度的坐标值设置为缺省值
    id_threshold_sta[["level","time","dtime"]] = meteva.base.IV

    #根据闪电判断雷暴大风的闪电观测必要条件
    sta_ob_01 = meteva.method.point_to_area.p2p_vto01(sta_ob_linghtning_count,threshold= id_threshold_sta)

    return  sta_ob_01

if __name__ == "__main__":
    meteva.base.set_io_config(r"H:\test_data\ip_port.txt") #设置本地环境下数据服务器的账号信息

    #  读取自动站的站号和经纬度信息的文件,其中需包含站号，经度、纬度和站点类型，读入后分别存储在id,lon,lat,type列
    # type列中国家站取值需为1，其它站取值不为1
    #  实际业务中站点信息文件以预报司下发的为准
    station_file = r"H:\resource\infomation-dat\station_3w.dat"
    station_all = meteva.base.read_stadata_from_micaps3(station_file,data_name= "type")

    time_start = datetime.datetime(2020,10,31,2,0)
    time_end = datetime.datetime(2020,10,31,20,0)


    #暂时不能确定实际竞赛中用到的观测数据是原始的闪电数据，还是处理好的站点形式的闪电数据，亦或者是处理好的雷暴大风实况数据
    #以下只能假定检从最原始的闪电数据处理开始，编写数据收集的示例代码，以供参考

    ###读取收集micaps分布式服务器上逐小时闪电观测数据
    dir_ob = r"mdfs:///SURFACE/LIGHTNING_1H/YYYYMMDDHH0000.000"
    sta_list = []
    time0 = time_start
    while time0 <= time_end:
        path = meteva.base.get_path(dir_ob,time0)
        meteva.base.print_gds_file_values_names(path)
        sta = meteva.base.read_stadata_from_gds(path,meteva.base.gds_element_id.电流强度_闪电定位,
                                                time = time0,dtime = 0,level = 0,data_name = "lightning",show = True)
        sta_list.append(sta)
        time0 += datetime.timedelta(hours = 1)
    lightning_ob = pd.concat(sta_list,axis = 0)  #数据拼接

    sta1 = edition_2020_1_lightning_ob_to_count(lightning_ob,station_all)

    ###读取收集micaps分布式服务器上逐小时最大风速观测数据
    dir_ob = r"mdfs:///SURFACE/MAX_WIND/YYYYMMDDHH0000.000"
    sta_list = []
    time0 = time_start
    while time0 <= time_end:
        path = meteva.base.get_path(dir_ob,time0)
        meteva.base.print_gds_file_values_names(path)
        sta = meteva.base.read_stadata_from_gds(path,meteva.base.gds_element_id.极大风速,
                                                time = time0,dtime = 0,level = 0,data_name = "maxwind",show = True)
        sta_list.append(sta)
        time0 += datetime.timedelta(hours = 1)
    max_wind_ob = pd.concat(sta_list,axis = 0)  #数据拼接



    # 收集中央台预报数据，格点的数值为1代表预报强对流事件发生，0代表预报强对流事件不发生
    dir_na = r"O:\data\grid\scmoc\YYYYMMDD\YYMMDDHH.TTT.nc"
    sta_list = []
    time0 = time_start
    while time0 < time_end:
        for dh in range(1, 13):  # 预报时效包含 1,2，3，。。。，12
            path = meteva.base.get_path(dir_na, time0, dh)
            grd = meteva.base.read_griddata_from_nc(path,time=time0,dtime = dh,level = 0,data_name="scmoc",show=True)
            if grd is not None:
                sta = meteva.base.interp_gs_linear(grd, station_all)
                sta_list.append(sta)
        time0 += datetime.timedelta(hours=6)  #起报时间包含 02,08,14,20
    scmoc_sta_all = pd.concat(sta_list, axis=0)  # 数据拼接


    # 收集省台预报数据，格点的数值为1代表预报强对流事件发生，0代表预报强对流事件不发生
    dir_pr = r"O:\data\grid\smerge\YYYYMMDD\YYMMDDHH.TTT.nc"
    sta_list = []
    time0 = time_start
    while time0 < time_end:
        for dh in range(1, 13):  # 预报时效包含 1,2，3，。。。，12
            path = meteva.base.get_path(dir_pr, time0, dh)
            grd = meteva.base.read_griddata_from_nc(path,time=time0,dtime = dh,level = 0,data_name="smerge",show=True)
            if grd is not None:
                sta = meteva.base.interp_gs_linear(grd, station_all)
                sta_list.append(sta)
        time0 += datetime.timedelta(hours=6)   #起报时间包含 02,08,14,20
    smerge_sta_all = pd.concat(sta_list, axis=0)  # 数据拼接

    #读取城镇预报，原则上从其它文件中读取，但为了简化示例，此处从网格预报中提取
    pcapital_id = [54511]
    scmoc_sta_pcapital = meteva.base.in_id_list(scmoc_sta_all,pcapital_id)
    smerge_sta_pcapital = meteva.base.in_id_list(smerge_sta_all,pcapital_id)

    #调用计算模块计算相关技巧
    spo, sfa, spo_pcapital, sfa_pcapital = edtion_2020_1(lightning_ob,station_all,max_wind_ob,[scmoc_sta_all,smerge_sta_all],
                                                     [scmoc_sta_pcapital,smerge_sta_pcapital])



from meteva.method.space.baddeley_binary_image_metric.locmeasures2d_spatial_vx import locmeasures2d_spatial_vx
from meteva.method.space.baddeley_binary_image_metric.lib.make_spatialVx import make_spatialVx
#import pyreadr


def locmeasures2d_default(grd_ob, grd_fo, thresholds, which_stats=None, k=None, alpha=None, bdconst=None, p=None):
    # ph 并没有进行计算
    if p is None:
        p = [2]
    if alpha is None:
        alpha = [0.1]
    if which_stats is None:
        which_stats = ["bdelta", "haus", "qdmapdiff", "med", "msd", "ph", "fom"]
    if grd_fo.shape != grd_ob.shape:
        raise Exception("locmeasures2d: dim of observed field (", grd_ob.shape, ") must equal dim of forecast field (",
                        grd_fo.shape, ")")
    obj = make_spatialVx(grd_ob=grd_ob, grd_fo=grd_fo, loc=None, thresholds=thresholds)
    out = locmeasures2d_spatial_vx(input_object=obj, which_stats=which_stats, k=k, alpha=alpha, bdconst=bdconst, p=p)
    return out


if __name__ == '__main__':
    # geom000 = pyreadr.read_r('./data/geom000.Rdata')['geom000']
    # geom001 = pyreadr.read_r('./data/geom001.Rdata')['geom001']

    import meteva.base as meb
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    path_fo_27 = r'H:\test_data\input\mem\mode\ec\rain03\20072508.027.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo_03 = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    grd_fo_27 = meb.read_griddata_from_nc(path_fo_27, grid=grid1, time="2020072508", dtime=27, data_name="ECMWF")

    obs_array = grd_ob.values.squeeze()
    fst_array = grd_fo_03.values.squeeze()

    look = locmeasures2d_default(obs_array, fst_array, [0.01, 50.01, 99.01], k=[4, 0.975], alpha=0.1)
    print("j")



import pkg_resources
import re
import meteva
import pandas as pd

def fuzzyfinder(input_str, collection):
    suggestions = []
    pattern = '.*?'.join(input_str)    # Converts 'djm' to 'd.*?j.*?m'
    regex = re.compile(pattern)         # Compiles a regex.
    for item in collection:
        match = regex.search(item)      # Checks if the current item matches the regex.
        if match:
            suggestions.append((len(match.group()), match.start(), item))
    return [x for _, _, x in sorted(suggestions)]

def muti_strs_finder(input_str, collection):
    strs = input_str.split()
    input_collection = collection
    output_str_list = []
    for i in range(len(strs)):
        output_str_list = fuzzyfinder(strs[i], input_collection)
        if len(output_str_list) ==0:
            if i==0:
                return output_str_list
        else:
            input_collection = output_str_list
    return input_collection

def get_station_id_name_dict(id_name_list_file):
    file1 = None
    try:
        file1 = open(id_name_list_file, encoding="GBK")
        str0 = file1.read()
    except:
        try:
            file1 = open(id_name_list_file,encoding="UTF-8")
            str0 = file1.read()
        except:
            pass
    if file1 is not None:
        file1.close()
        str1 = str0.split("\n")
        station_id_name_dict = {}
        for i in range(len(str1)):
            strs_line = str1[i].split(" ")
            if len(strs_line) ==3:
                values = strs_line[0]
                strs_int = ""
                for i in range(len(values)):
                    strs = values[i]
                    for s in strs:
                        if s.isdigit():
                            strs_int += s
                        else:
                            strs_int += str(ord(s))
                if(len(strs_int) >0):
                    key =  int(strs_int)
                    if not key in station_id_name_dict.keys():
                        value1 = strs_line[1]+"_"+strs_line[2]
                        station_id_name_dict[key] = value1
    else:
        print("warming ：站点名称文件"+id_name_list_file+"读取失败,在后续的产品中不能显示站点中文名称")
        station_id_name_dict = {}
    return station_id_name_dict

station_id_name_dict = get_station_id_name_dict(pkg_resources.resource_filename('meteva', "resources/stations/station_id_pro_county.txt"))
station_name_id_dict = dict(zip(station_id_name_dict.values(),station_id_name_dict.keys()))


def add_station_id_name_dict(id_name_dict):
    for id in id_name_dict.keys():
        name = id_name_dict[id]
        meteva.base.station_id_name_dict[id] = name
        meteva.base.station_name_id_dict[name] = id

def find_station_id_by_city_name(input_strs):
    ele_names = muti_strs_finder(input_strs,station_name_id_dict)
    names_ids = {}
    for names in ele_names:
        names_ids[names] = station_name_id_dict[names]
        #print(names + " : " + str(names_ids[names]))
    return names_ids


def get_station_format_province_set(id_list):
    pro_names = []
    no_province_ids = []
    for id in id_list:
        if id in station_id_name_dict.keys():
            str1 = station_id_name_dict[id]
            str1s = str1.split("_")
            pro_names.append(str1s[0])
        else:
            pro_names.append("未区分")
            no_province_ids.append(id)

    df = pd.DataFrame({
        "id":id_list,
        "province_name":pro_names
    })

    sta = meteva.base.sta_data(df)

    return sta


#!/usr/bin/python3.6
# -*- coding:UTF-8 -*-
import math
from copy import deepcopy
#创建一个类ctl,并初始化一些变量。
import os
import numpy as np
import datetime
import re


def read_ctl(ctl_filename,dtime_dim = None,dtime_start = 0):
    #print(ctl_filename)
    if os.path.exists(ctl_filename):
        ctl = {}
        file = open(ctl_filename, 'r')
        line = file.read()
        strs_all = line.split()
        file.close()

        file = open(ctl_filename, 'r')
        line = file.readline()
        strs = line.split()

        if strs[1].__contains__('^'):
            ctl["data_path"] = os.path.dirname(ctl_filename) + "/" + strs[1][1:]
        else:
            ctl["data_path"] = strs[1]
        #print( ctl["data_path"])
        index_zdef = 0
        while line:
            strs = line.split()
            if len(strs) >0:
                index_zdef += len(strs)
                if strs[0].upper() == "PDEF":
                    ctl["pdef"] = {}
                    ctl["pdef"]["nx"] = int(strs[1])
                    ctl["pdef"]["ny"] = int(strs[2])
                    ctl["pdef"]["type"] = strs[3]
                    ctl["pdef"]["model_lat"] = float(strs[4])
                    ctl["pdef"]["model_lon"] = float(strs[5])
                    ctl["pdef"]["model_i"] = float(strs[6])
                    ctl["pdef"]["model_j"] = float(strs[7])
                    ctl["pdef"]["proj_slat"] = float(strs[8])
                    ctl["pdef"]["proj_elat"] = float(strs[9])
                    ctl["pdef"]["proj_mlon"] = float(strs[10])
                    ctl["pdef"]["dx"] = float(strs[11])
                    ctl["pdef"]["dy"] = float(strs[12])

                if strs[0].upper() == "XDEF":
                    if strs[2].lower() == "linear":
                        nlon = int(strs[1])
                        slon = float(strs[3])
                        dlon = float(strs[4])
                        elon = slon + (nlon -1) * dlon
                        ctl["xdef"] = (np.arange(nlon) * dlon + slon).tolist()
                        ctl["glon"] = [slon,elon,dlon]
                        ctl["nlon"] =nlon
                if strs[0].upper() == "YDEF":
                    if strs[2].lower() == "linear":
                        nlat = int(strs[1])
                        slat = float(strs[3])
                        dlat = float(strs[4])
                        elat = slat +(nlat-1)*dlat
                        ctl["ydef"] = (np.arange(nlat) * dlat + slat).tolist()
                        ctl["glat"] = [slat, elat, dlat]
                        ctl["nlat"] = nlat
                if strs[0].upper() == "ZDEF":
                    if strs[2].upper()=="LINEAR":
                        ctl["nlevel"] = int(strs[1])
                        levels = []
                        for ii in range(ctl["nlevel"]):

                            levels.append(float(strs[3]) + ii * float(strs[4]))
                        ctl["zdef"] = levels
                    else:
                        nlevel =  int(strs[1])
                        ctl["nlevel"] =nlevel
                        levels = []
                        if len(strs) > 3:
                            for i in range(nlevel):
                                levels.append(float(strs_all[index_zdef + i - nlevel]))
                        else:
                            for i in range(nlevel):
                                levels.append(float(strs_all[index_zdef + i]))

                        ctl["zdef"] = levels
                        ctl["nlevel"] = len(levels)


                if strs[0].upper() == "TDEF":
                    ntime = int(strs[1])
                    ctl["ntime"] = ntime
                    if strs[2].upper() == "LINEAR":
                        time_str = strs[3]
                        year = int(strs[3][-4:])
                        #print(strs[3])
                        mm_str = strs[3][-7:-4].lower()
                        dict1 = {"jan": 1, "feb": 2, "mar": 3, "apr": 4, "may": 5, "jun": 6,
                                 "jul": 7, "aug": 8, "sep": 9, "oct": 10, "nov": 11, "dec": 12}
                        mm = dict1[mm_str]
                        zindex = -1
                        if time_str.find("z") >= 0:
                            zindex = time_str.find("z")
                        elif time_str.find("Z") >= 0:
                            zindex = time_str.find("Z")

                        if zindex >= 0:
                            dd = int(time_str[zindex + 1:-7])
                            hh = int(time_str[0:zindex])
                        else:
                            dd = int(time_str[0:-7])
                            hh = 0
                        time_begin = datetime.datetime(year, mm, dd, hh, 0)
                        dtime_str = strs[4]
                        dt = int(dtime_str[0:-2])
                        dict2 = {"mn": "min", "hr": "H", "dy": "D", "mo": "M", "yr": "Y"}
                        dt_unit = dtime_str[-2:].lower()

                        if dt_unit == "mn":
                            time_end = time_begin + datetime.timedelta(minutes=dt * (ntime - 1))
                            if dt>=60:
                               dt_unit ="h"
                               dt  = int(dt/ 60)
                            else:
                                dt_unit = "min"
                                dt = dt
                        elif dt_unit== "hr":
                            dt_unit = "h"
                            time_end = time_begin + datetime.timedelta(hours=dt * (ntime - 1))
                        elif dt_unit== "dy":
                            dt_unit = "h"
                            time_end = time_begin + datetime.timedelta(days=dt * (ntime - 1))
                            dt *= 24
                        else:
                            time_end = None
                            print("暂时还不能支持逐月或逐年数据的读取")

                        if dtime_dim is None:
                            ctl["gtime"] = [time_begin, time_end, str(dt) + dt_unit]
                            ctl["dtime_list"] = [0]
                        else:
                            time_begin0 = time_begin - datetime.timedelta(hours=dtime_start)
                            ctl["gtime"] = [time_begin0]
                            ctl["dtime_list"] = (np.arange(ntime) * dt + dtime_start).tolist()

                    else:
                        print("TDEF 不是线性数组的情况暂不能支持")


                if (strs[0].upper() == "EDEF"):
                    nensemble = int(strs[1])
                    ctl["edef"] = np.arange(nensemble).tolist()
                    ctl["nensemble"] = nensemble

                if strs[0].upper() == "VARS":
                    nvar = int(strs[1])
                    ctl["vars"] = []
                    cumulate = 0
                    for v in range(nvar):
                        line0 = file.readline()
                        line = line0.strip()
                        #strs = line.split()
                        strs = re.split("\s+|,",line)
                        onev = {}
                        onev["name"] = strs[0]
                        nlevel = int(strs[1])
                        if nlevel == 0:
                            nlevel = 1
                        onev["nlevel"] = nlevel
                        onev["type"] = int(strs[2])
                        onev["discription"] = strs[3]
                        onev["start_bolck_index"] = cumulate
                        cumulate += nlevel
                        ctl["vars"].append(onev)
                    ctl["cumulate_levels"] = cumulate

            line = file.readline()
        if "edef" not in ctl.keys():
            ctl["edef"] = [0]
            ctl["nensemble"] = 1
        if "ntime" not in ctl.keys():
            ctl["ntime"] = 1
        return ctl
    else:
        return None




import numpy as np
import math

import numpy.fft


def fft2d(x, bigdim=None, inverse=False):
    is_null = False
    if bigdim is None:
        is_null = True
    x = np.array(x)
    out = {}
    xdim = x.shape
    if is_null:
        if xdim[0] <= 1024:
            x_index = 2 ** (math.ceil(math.log2(xdim[0])))
        else:
            x_index = math.ceil(xdim[0] / 512) * 512
        if xdim[1] <= 1024:
            y_index = 2 ** math.ceil(math.log2(xdim[1]))
        else:
            y_index = math.ceil(xdim[1] / 512) * 512
        bigdim = (x_index, y_index)
    hold = np.full(bigdim, 0 + 0j)
    hold[0: xdim[0], 0: xdim[1]] = x
    if not inverse:
        hold = numpy.fft.fft2(hold)[0: xdim[0], 0: xdim[1]]
    else:
        hold = hold.size * np.fft.ifft(hold.flatten()).reshape(hold.shape)[0: xdim[0], 0: xdim[1]]
    if is_null:
        out["fft"] = hold
        out["bigdim"] = bigdim
    else:
        out = hold
    return out




from . import score
from . import table
from . import plot
from .score import hk,hk_tcof,hss,hss_tcof,tc,tcof,accuracy,accuracy_tc,accuracy_tcof,hfmc_multi,ts_multi,ets_multi,bias_multi,far_multi,mr_multi
from .score import hfmc_grade,ts_grade,ets_grade,bias_grade,far_grade,mr_grade,seeps_ctable,seeps,seeps_skill
from .score import pod_grade,pofd_grade,sr_grade,pod_multi,pofd_multi,sr_multi
from .table import contingency_table_multicategory,frequency_table
from .plot import frequency_histogram,performance_grade,performance_multi
from .score import murphy_score,get_s_array,murphy_ctable


import numpy as np
import matplotlib.pyplot as plt
import meteva
import math
import json

def error_boxplot(sta_ob_and_fos0,s = None, g = None, gll=None,
                  group_name_list=None,threshold = 2,save_dir=None,save_path = None,show = False,dpi = 200,title="误差综合分析图",
                  vmin  = None,vmax = None,spasify_xticks = None,sup_fontsize =10,width = None,height = None,json_dir = None,json_path = None,
                  **kwargs):
    '''

    :param sta_ob_and_fos0:
    :param s:
    :param g:
    :param gll:
    :param save_dir:
    :param group_name_list:
    :param threshold:
    :param show:
    :param title:
    :return:
    '''
    if s is not None:
        if g is not None:
            if g == "last_range" or g == "last_step":
                s["drop_last"] = False
            else:
                s["drop_last"] = True


    sta_ob_and_fos = meteva.base.sele_by_dict(sta_ob_and_fos0, s)
    if(len(sta_ob_and_fos.index) == 0):
        print("there is no data to verify")
        return
    sta_ob_and_fos_list, gll1 = meteva.base.fun.group(sta_ob_and_fos, g, gll)
    data_names = meteva.base.get_stadata_names(sta_ob_and_fos_list[0])
    if(len(data_names) ==1):
        print("error infomation: only one data column, can't caculate error")
        return

    if isinstance(title, list):
        if len(data_names) -1 != len(title):
            print("手动设置的title数目和要绘制的图形数目不一致")
            return



    if save_path is not None:
        if isinstance(save_path,str):
            save_path = [save_path]
        if len(data_names) -1 != len(save_path):
            print("手动设置的save_path数目和要绘制的图形数目不一致")
            return

    if json_path is not None:
        if isinstance(json_path,str):
            json_path = [json_path]
        if len(data_names) -1 != len(json_path):
            print("手动设置的json_path数目和要绘制的图形数目不一致")
            return

    if group_name_list is None:
        group_name_list = meteva.product.program.get_group_name(gll1)

    for v in range(len(data_names)-1):
        combineData = []
        boxgroup = []
        right_rate = []
        tcount = []
        me_list = []
        mae_list = []
        rmse_list = []
        max_list = []
        min_list = []
        if gll1 is None:
            gll1 = [[0]]
        picture_ele_dict = {}
        picture_ele_dict["box_data"] ={}

        for i in range(len(gll1)):
            dat = sta_ob_and_fos_list[i].values[:, 7+v] - sta_ob_and_fos_list[i].values[:, 6]
            #picture_ele_dict["box_data"][i] = dat.tolist()
            if vmax is None:
                max_list.append(np.max(dat))
            if vmin is None:
                min_list.append(np.min(dat))
            me_list.append(np.mean(dat))
            mae_list.append(np.mean(np.abs(dat)))
            rmse_list.append(np.sqrt(np.mean(np.power(dat,2))))
            tt = len(dat)
            index = np.where((dat <= threshold) & (dat >= -threshold))
            rcount = len(index[0])
            right_rate.append(rcount / tt)
            tcount.append(tt)
            combineData.append(dat)
            bg = np.ones(tt) * i
            boxgroup.append(bg)


        picture_ele_dict["me"] = me_list
        picture_ele_dict["mae"] = mae_list
        picture_ele_dict["rmse"] = rmse_list
        picture_ele_dict["correct_rate"] = right_rate
        picture_ele_dict["sample_count"] = tcount
        picture_ele_dict["xticklabels"] = group_name_list

        right_rate = np.array(right_rate)
        tcount = np.array(tcount)


        if vmin is None:
            vmin1 = np.min(np.array(min_list))
        else:
            vmin1 = vmin
        if vmax is None:
            vmax1 = np.max(np.array(max_list))
        else:
            vmax1 = vmax
        dmax = vmax1 - vmin1
        if vmin is None:
            if vmin1 < 0:
                vmin1 = vmin1 - 0.1 * dmax
        if vmax is None:
            vmax1 = vmax1 + 0.2 * dmax

        picture_ele_dict["vmin"] = vmin1
        picture_ele_dict["vmax"] = vmax1



        # 计算最大的横坐标字符串
        #max_str_len = 5
        #for k in range(1, len(group_name_list)):
        #    index = group_name_list[k]
        #    if not type(index) == str:
        #        index = str(index)
        #    indexs = index.split("\n")
        #    for i in range(len(indexs)):
        #        str1 = indexs[i]
        #        if max_str_len < len(str1):
        #            max_str_len = len(str1)

        width_axis_labels = meteva.base.plot_tools.caculate_axis_width(group_name_list, sup_fontsize, 1)
        width_wspace = 2
        width_one_subplot = width_axis_labels + width_wspace
        if width_one_subplot < 2: width_one_subplot = 2
        if width is None:
            width = max(4,min(width_one_subplot,8))

        spasify = 1
        if width_one_subplot > width:
            spasify = int(math.ceil(width_axis_labels / (width - width_wspace)))
        if spasify_xticks is not None:
            xticks_font = sup_fontsize * 0.8 * spasify_xticks * (width - width_wspace) / (width_axis_labels)
            spasify = spasify_xticks
        else:
            xticks_font = sup_fontsize * 0.8

        x = np.arange(1, len(tcount) + 1)
        xticks = x[::spasify]
        xticks_labels = group_name_list[::spasify]

        #if width is None:
        #    width = 0.5 + (1 + max_str_len) * 0.06 * len(group_name_list)
        #    if width < 4:width = 4
        #    if width >8:width = 8

        if height is None:
            height = 4

        fig = plt.figure(figsize=(width, height),dpi = dpi)

        grid_plt = plt.GridSpec(6, 1, hspace=0)
        plt.rcParams['xtick.direction'] = 'in'
        plt.rcParams['ytick.direction'] = 'in'
        ax1 = plt.subplot(grid_plt[0:5, 0])

        bplot = ax1.boxplot(combineData, boxgroup, patch_artist=True, sym='.',**kwargs)
        for i, item in enumerate(bplot["boxes"]):
            item.set_facecolor("lightblue")
        plt.subplots_adjust(left=0.5 / width, right=1 - 0.1 / width)
        plt.ylabel("误差值", fontsize=sup_fontsize *0.9)
        plt.yticks(fontsize = sup_fontsize * 0.8)
        plt.plot(x,me_list,'g',label = '平均误差',zorder = 3)
        plt.plot(x, mae_list, 'b', label='平均绝对误差',zorder = 3)
        plt.plot(x, rmse_list, 'r', label='均方根误差',zorder = 3)
        plt.hlines(0, 1, len(group_name_list), linewidth=1, color="k", linestyles="dashed")
        plt.hlines(threshold, 1, len(group_name_list), linewidth=1, color="k", linestyles="dashed")
        plt.hlines(-threshold, 1, len(group_name_list), linewidth=1, color="k", linestyles="dashed")
        plt.ylim(vmin1,vmax1)
        plt.legend(fontsize =sup_fontsize * 0.8,ncol = 3,loc = "upper center")


        if isinstance(title,list):
            title1 = title[v]
        else:
            title1 = meteva.product.program.get_title_from_dict(title, s, None, None,
                                                                data_names[v + 1])

        plt.title(title1, fontsize=14)

        ax2 = plt.subplot(grid_plt[5, 0])
        plt.rcParams['xtick.direction'] = 'in'
        plt.rcParams['ytick.direction'] = 'in'
        plt.bar(x, tcount,width=0.5,label = "样本数")
        plt.legend(fontsize=sup_fontsize * 0.5, loc="upper left")
        xlabel_1 = meteva.product.program.get_x_label(g)

        plt.xlabel(xlabel_1, fontsize=sup_fontsize *0.9)
        plt.xlim(0.5, len(tcount) + 0.5)
        plt.ylim(0, np.max(tcount) * 1.5)


        plt.xticks(xticks, xticks_labels,fontsize = xticks_font)
        plt.yticks(fontsize=sup_fontsize *0.8)
        plt.ylabel("样本数", fontsize=sup_fontsize * 0.9)
        ax2 = ax2.twinx()
        plt.plot(x, right_rate*100, "r",linestyle = "--", linewidth=1.5,label = "准确率")
        plt.legend(fontsize=sup_fontsize * 0.5, loc="upper right")
        plt.ylim(0, 100)
        plt.yticks([0,25,50,75,100],fontsize=sup_fontsize *0.8)
        plt.ylabel("准确率(%)", fontsize = sup_fontsize * 0.9)

        save_path1 = None
        if save_path is None:
            if save_dir is None:
                show = True
            else:
                save_path1 = save_dir + "/" + data_names[v + 1] + ".png"
        else:
            save_path1 = save_path[v]
        if save_path1 is not None:
            meteva.base.tool.path_tools.creat_path(save_path1)
            plt.savefig(save_path1,bbox_inches='tight')
            print("图片已保存至" + save_path1)
        if show:
            plt.show()
        plt.close()


        json_path1 = None
        if json_path is None:
            if json_dir is None:
                pass
            else:
                json_path1 = json_dir + "/" + data_names[v + 1] + ".json"
        else:
            json_path1 = json_path[v]
        if json_path1 is not None:
            meteva.base.tool.path_tools.creat_path(json_path1)
            file = open(json_path1, "w")
            json.dump(picture_ele_dict, file)
            print("have printed pictrue elements to " + json_path1)



def error_boxplot_abs(sta_ob_and_fos0,s = None, g = None, gll=None,
                  group_name_list=None,threshold = 2,save_dir=None,save_path = None,show = False,dpi = 200,title="绝对误差综合分析图",
                    vmin = None, vmax = None, spasify_xticks = None, sup_fontsize = 10, width = None, height = None
                      ,json_dir = None,json_path = None,**kwargs):

    if s is not None:
        if g is not None:
            if g == "last_range" or g == "last_step":
                s["drop_last"] = False
            else:
                s["drop_last"] = True

    sta_ob_and_fos = meteva.base.sele_by_dict(sta_ob_and_fos0, s)
    if(len(sta_ob_and_fos.index) == 0):
        print("there is no data to verify")
        return
    data_names = meteva.base.get_stadata_names(sta_ob_and_fos)
    if(len(data_names) ==1):
        print("error infomation: only one data column, can't caculate error")
        return

    if save_path is not None:
        if isinstance(save_path,str):
            save_path = [save_path]
        if len(data_names) -1 != len(save_path):
            print("手动设置的save_path数目和要绘制的图形数目不一致")
            return

    if isinstance(title, list):
        if len(data_names) -1 != len(title):
            print("手动设置的title数目和要绘制的图形数目不一致")
            return

    if json_path is not None:
        if isinstance(json_path,str):
            json_path = [json_path]
        if len(data_names) -1 != len(json_path):
            print("手动设置的json_path数目和要绘制的图形数目不一致")
            return

    sta_ob_and_fos_list, gll1 = meteva.base.fun.group(sta_ob_and_fos, g, gll)
    if group_name_list is None:
        group_name_list = meteva.product.program.get_group_name(gll1)
    if gll1 is None:
        gll1 = [[0]]
    for v in range(len(data_names)-1):
        combineData = []
        boxgroup = []
        right_rate = []
        tcount = []
        maxlist = []

        me_list = []
        mae_list = []
        rmse_list = []
        max_list = []

        picture_ele_dict = {}
        picture_ele_dict["box_data"] ={}

        for i in range(len(gll1)):
            dat = sta_ob_and_fos_list[i].values[:, 7+v] - sta_ob_and_fos_list[i].values[:, 6]
            me_list.append(np.mean(dat))
            dat = np.abs(dat)
            if vmax is None:
                max_list.append(np.max(dat))
            mae_list.append(np.mean(dat))
            rmse_list.append(np.sqrt(np.mean(dat * dat)))

            maxlist.append(np.max(dat))
            tt = len(dat)
            index = np.where(dat <= threshold)
            rcount = len(index[0])
            right_rate.append(rcount / tt)
            tcount.append(tt)
            combineData.append(dat)
            bg = np.ones(tt) * i
            boxgroup.append(bg)
        right_rate = np.array(right_rate)
        tcount = np.array(tcount)
        maxarray = np.array(maxlist)
        maxerror = np.max(maxarray)

        picture_ele_dict["me"] = me_list
        picture_ele_dict["mae"] = mae_list
        picture_ele_dict["rmse"] = rmse_list
        picture_ele_dict["correct_rate"] = right_rate
        picture_ele_dict["sample_count"] = tcount
        picture_ele_dict["xticklabels"] = group_name_list

        if vmin is None:
            vmin1 = np.min(np.array(me_list))
        else:
            vmin1 = vmin
        if vmax is None:
            vmax1 = np.max(np.array(max_list))
        else:
            vmax1 = vmax
        dmax = vmax1 - vmin1
        if vmin is None:
            if vmin1 < 0:
                vmin1 = vmin1 - 0.1 * dmax
        if vmax is None:
            vmax1 = vmax1 + 0.2 * dmax

        picture_ele_dict["vmin"] = vmin1
        picture_ele_dict["vmax"] = vmax1



        # 计算最大的横坐标字符串
        width_axis_labels = meteva.base.plot_tools.caculate_axis_width(group_name_list, sup_fontsize, 1)
        width_wspace = 2
        width_one_subplot = width_axis_labels + width_wspace
        if width_one_subplot < 2: width_one_subplot = 2
        if width is None:
            width = max(4,min(width_one_subplot,8))

        spasify = 1
        if width_one_subplot > width:
            spasify = int(math.ceil(width_axis_labels / (width - width_wspace)))
        if spasify_xticks is not None:
            xticks_font = sup_fontsize * 0.8 * spasify_xticks / spasify
            spasify = spasify_xticks
        else:
            xticks_font = sup_fontsize * 0.8


        x = np.arange(1, len(tcount) + 1)
        xticks = x[::spasify]
        xticks_labels = group_name_list[::spasify]
        if height is None:
            height = 4

        fig = plt.figure(figsize=(width, height),dpi = dpi)
        grid_plt = plt.GridSpec(5, 1, hspace=0)
        x = np.arange(1, len(tcount) + 1)
        plt.rcParams['xtick.direction'] = 'in'
        plt.rcParams['ytick.direction'] = 'in'
        ax1 = plt.subplot(grid_plt[0:4, 0])
        bplot = ax1.boxplot(combineData, boxgroup,  patch_artist=True, sym='.',**kwargs)
        for i, item in enumerate(bplot["boxes"]):
            item.set_facecolor("lightblue")
        plt.subplots_adjust(left=0.5 / width, right=1 - 0.1 / width)
        plt.ylabel("误差绝对值", fontsize=sup_fontsize *0.9)

        plt.yticks(fontsize=sup_fontsize * 0.8)
        plt.ylim(vmin1,vmax1)
        plt.plot(x,me_list,'g',label = '平均误差',zorder = 3)
        plt.plot(x, mae_list, 'b', label='平均绝对误差',zorder = 3)
        plt.plot(x, rmse_list, 'r', label='均方根误差',zorder = 3)
        plt.legend(fontsize=sup_fontsize * 0.8, ncol=3, loc="upper center")

        if isinstance(title,list):
            title1 = title[v]
        else:
            title1 = meteva.product.program.get_title_from_dict(title, s, None, None,
                                                                data_names[v + 1])


        plt.title(title1, fontsize=sup_fontsize)

        plt.hlines(threshold, 1, len(group_name_list), linewidth=1, color="k", linestyles="dashed")
        plt.hlines(0, 1, len(group_name_list), linewidth=1, color="k", linestyles="dashed")
        ax2 = plt.subplot(grid_plt[4, 0])
        plt.rcParams['xtick.direction'] = 'in'
        plt.rcParams['ytick.direction'] = 'in'
        plt.bar(x, tcount,width=0.5,label = "样本数")
        plt.legend(fontsize=sup_fontsize * 0.5, loc="upper left")
        xlabel_1 = meteva.product.program.get_x_label(g)
        plt.xlabel(xlabel_1, fontsize=sup_fontsize *0.9)
        plt.xlim(0.5, len(tcount) + 0.5)
        plt.ylim(0, np.max(tcount) * 1.5)
        plt.xticks(xticks, xticks_labels,fontsize = xticks_font)
        plt.yticks(fontsize=sup_fontsize * 0.8)
        plt.ylabel("样本数", fontsize=sup_fontsize * 0.9)
        ax3 = ax2.twinx()
        plt.plot(x, right_rate*100, "r",linestyle = "--", linewidth=1.5,label = "准确率")
        plt.legend(fontsize=sup_fontsize * 0.5, loc="upper right")
        plt.ylim(0, 100)
        plt.yticks([0,25,50,75,100],fontsize=sup_fontsize *0.8)
        plt.ylabel("准确率(%)", fontsize=sup_fontsize * 0.9)

        save_path1 = None
        if save_path is None:
            if save_dir is None:
                show = True
            else:
                save_path1 = save_dir + "/" + data_names[v + 1] + ".png"
        else:
            save_path1 = save_path[v]
        if save_path1 is not None:
            meteva.base.tool.path_tools.creat_path(save_path1)
            plt.savefig(save_path1,bbox_inches='tight')
            print("图片已保存至" + save_path1)
        if show:
            plt.show()
        plt.close()


        json_path1 = None
        if json_path is None:
            if json_dir is None:
                pass
            else:
                json_path1 = json_dir + "/" + data_names[v + 1] + ".json"
        else:
            json_path1 = json_path[v]
        if json_path1 is not None:
            meteva.base.tool.path_tools.creat_path(json_path1)
            file = open(json_path1, "w")
            json.dump(picture_ele_dict, file)
            print("have printed pictrue elements to " + json_path1)



import numpy as np


def ampstats(x, xhat, only_nonzero=False):
    x = np.array(x)
    xhat = np.array(xhat)
    if only_nonzero:
        idy = (xhat != 0)
        idx = (x != 0)
    else:
        idy = np.full(xhat.shape, True)
        idx = np.full(x.shape, True)
    m1 = np.nanmean(xhat[idy])
    v1 = np.nanvar(xhat[idy])
    m2 = np.nanmean(x[idx])
    v2 = np.nanvar(x[idx])
    if not only_nonzero:
        ind = (~np.isnan(xhat)) & (~np.isnan(x))
        v12 = np.cov(xhat[ind].flatten(), x[ind].flatten())
        v12 = v12[0][1]
    else:
        v12 = np.nan
    return {"mean_fcst": m1, "mean_vx": m2, "var_fcst": v1, "var_vx": v2, "cov": v12}



from meteva.method.space.fuzzy_logic.lib.datagrabber_spatialVx import datagrabber_spatialVx
from meteva.method.space.fuzzy_logic.lib.thresholder import thresholder


def thresholder_spatialVx(x, func_type=None, th=1, rule=">=", replace_with=0, time_point=1, obs=1,
                          model=1, **args):
    if func_type is None:
        func_type = ["binary", "replace.below"]
    dat = datagrabber_spatialVx(x, time_point=time_point, obs=obs, model=model)
    x_field = dat['X']
    xhat_field = dat['Xhat']
    u = x['thresholds']
    u1 = u[0][th]
    u2 = u[1][th]
    out1 = thresholder(x_field, func_type=func_type, th=u1, rule=rule, replace_with=replace_with)
    out2 = thresholder(xhat_field, func_type=func_type, th=u2, rule=rule, replace_with=replace_with)
    return {'X': out1, 'Xhat': out2}



import numpy  as np
from meteva.method.space.fuzzy_logic.lib.vxstats import vxstats


def pragmatic2dfun(s_py, ix, m_ix=None, subset=None):
    bs = vxstats(s_py, ix, which_stats="mse", subset=subset)['mse']
    if subset is None:
        if m_ix is None:
            m_ix = np.mean(ix)
        denom = np.sum((m_ix - ix) ** 2) / np.sum(~np.isnan(ix))
    else:
        if m_ix is None:
            m_ix = np.mean(ix[subset])
        denom = np.mean((m_ix - ix[subset]) ** 2)

    bss = 1 - bs / denom
    return {'bs': bs, 'bss': bss}



import meteva
import datetime

def acc_climate_pre(input_dir,output_path):
    time1 = datetime.datetime(2020,1,1,0)
    grd_list = []
    for i in range(732):
        path = input_dir + r"\\"+str(i+1)+".grib"
        grd = meteva.base.read_griddata_from_grib(path)
        time2 = time1 + datetime.timedelta(hours=12*i)
        meteva.base.set_griddata_coords(grd,gtime=[time2])
        grd_list.append(grd)
    grd_all = meteva.base.concat(grd_list)
    meteva.base.write_griddata_to_nc(grd_all,save_path=output_path)


if __name__ == "__main__":
    input_dir = r"H:\test_data\input\mem\era-dailyclim\t700_mean"
    output_path = r"H:\test_data\input\mem\era-dailyclim\t700_mean.nc"
    acc_climate_pre(input_dir,output_path)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jan 27 18:29:57 2022

@author: tangbuxing
"""
import numpy as np


def CI_fun(CI_field, est_field, replacement=np.nan):
    test = est_field[0.5 * CI_field < abs(est_field)] = replacement
    return test


def inside(DF):
    # result = CI_fun(DF['Upper'] - DF['Lower'], DF['Estimate'])
    aa = DF['Upper'] - DF['Lower']
    bb = DF['Estimate']
    result = 0.5 * aa < abs(bb)  #满足条件代表平均值是显著偏离1的
    return result


def sig_coverage(DF):
    out = {}
    tmp = inside(DF)
    #sum(DF['Estimate'] == None) =0 表示df中的内容是误差均值，否则表示其它暂无支持的统计内容
    # out 是表示均值显著的占比。
    out = sum(tmp[tmp == 1]) / len(tmp) - sum(DF['Estimate'] == None)
    return out



